@@ -2,7 +2,9 @@
 #define	authldap_h
 
 /*
-** Copyright 1998 - 1999 Double Precision, Inc.  See COPYING for
+** $Id: authldap.h,v 1.5 2003/05/01 21:53:30 mrsam Exp $
+**
+** Copyright 1998 - 2003 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
@@ -15,7 +17,7 @@
 
 struct authinfo;
 
-int authldapcommon(const char *,
+int authldapcommon(const char *, const char *,
 	const char *, int (*)(struct authinfo *, void *), void *);
 
 void authldapclose();
