@@ -16,11 +16,13 @@ struct authmysqluserinfo {
 	char *home;
 	char *maildir;
 	char *quota;
+	char *options;
 	uid_t uid;
 	gid_t gid;
 	} ;
 
-extern struct authmysqluserinfo *auth_mysql_getuserinfo(const char *);
+extern struct authmysqluserinfo *auth_mysql_getuserinfo(const char *,
+							const char *);
 extern void auth_mysql_cleanup();
 
 extern int auth_mysql_setpass(const char *, const char *);
