@@ -1,5 +1,5 @@
 /*
-** Copyright 2000-2001 Double Precision, Inc.  See COPYING for
+** Copyright 2000-2004 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
@@ -16,6 +16,7 @@
 #include	"authmysql.h"
 #include	"authmysqlrc.h"
 #include	"auth.h"
+#include	"debug.h"
 
 /* siefca@pld.org.pl */
 #define		MAX_SUBSTITUTION_LEN	32
@@ -24,7 +25,7 @@
 #define		SV_BEGIN_LEN		((sizeof(SV_BEGIN_MARK))-1)
 #define		SV_END_LEN		((sizeof(SV_END_MARK))-1)
 
-static const char rcsid[]="$Id: authmysqllib.c,v 1.16 2002/01/10 04:41:41 mrsam Exp $";
+static const char rcsid[]="$Id: authmysqllib.c,v 1.36 2004/07/24 03:36:52 mrsam Exp $";
 
 /* siefca@pld.org.pl */
 struct var_data {			
@@ -138,6 +139,7 @@ const	char *p;
 			
 		if (mysql_ping(mysql) == 0) return (0);
 
+		dprintf("authmysqllib: mysql_ping failed, connection lost");
 		mysql_close(mysql);
 		mysql=0;
 	}
@@ -159,24 +161,24 @@ const	char *p;
 		server_opt=(unsigned int) atol(p);
 	}
 
-	if (!server)
+	if (!server && !server_socket)
 	{
-		fprintf(stderr, "authmysql: MYSQL_SERVER not set in "
-			AUTHMYSQLRC ".\n");
+		err("authmysql: MYSQL_SERVER nor MYSQL_SOCKET set in"
+			AUTHMYSQLRC ".");
 		return (-1);
 	}
 
 	if (!userid)
 	{
-		fprintf(stderr, "authmysql: MYSQL_USERNAME not set in "
-			AUTHMYSQLRC ".\n");
+		err("authmysql: MYSQL_USERNAME not set in "
+			AUTHMYSQLRC ".");
 		return (-1);
 	}
 
 	if (!database)
 	{
-		fprintf(stderr, "authmysql: MYSQL_DATABASE not set in "
-			AUTHMYSQLRC ".\n");
+		err("authmysql: MYSQL_DATABASE not set in "
+			AUTHMYSQLRC ".");
 		return (-1);
 	}
 
@@ -190,11 +192,17 @@ const	char *p;
 #else
 	mysql=mysql_connect(&mysql_buf, server, userid, password);
 #endif
-	if (!mysql)	return (-1);
+	if (!mysql)
+	{
+		err("failed to connect to mysql server (server=%s, userid=%s)",
+			server ? server : "<null>",
+			userid ? userid : "<null>");
+		return (-1);
+	}
 
 	if (mysql_select_db(mysql, database))
 	{
-		fprintf(stderr, "authmysql: mysql_select_db(%s) error: %s\n",
+		err("authmysql: mysql_select_db(%s) error: %s",
 			database, mysql_error(mysql));
 		mysql_close(mysql);
 		mysql=0;
@@ -212,13 +220,35 @@ void auth_mysql_cleanup()
 	}
 }
 
-static struct authmysqluserinfo ui={0, 0, 0, 0, 0, 0, 0};
+static struct authmysqluserinfo ui={0, 0, 0, 0, 0, 0, 0, 0};
+
+static void initui()
+{
+
+	if (ui.username)
+		free(ui.username);
+	if (ui.cryptpw)
+		free(ui.cryptpw);
+	if (ui.clearpw)
+		free(ui.clearpw);
+	if (ui.home)
+		free(ui.home);
+	if (ui.maildir)
+		free(ui.maildir);
+	if (ui.quota)
+		free(ui.quota);
+	if (ui.fullname)
+		free(ui.fullname);
+	if (ui.options)
+		free(ui.options);
+	memset(&ui, 0, sizeof(ui));
+}
 
 static void append_username(char *p, const char *username,
 			    const char *defdomain)
 {
 	for (strcpy(p, username); *p; p++)
-		if (*p == '"' || *p == '\\' ||
+		if (*p == '\'' || *p == '"' || *p == '\\' ||
 		    (int)(unsigned char)*p < ' ')
 			*p=' ';	/* No funny business */
 	if (strchr(username, '@') == 0 && defdomain && *defdomain)
@@ -233,23 +263,23 @@ struct var_data *vdp;
 
 	if (!begin || !vdt) /* should never happend */
 	{
-		fprintf (stderr, "authmysql: critical error while "
-				 "parsing substitution variable\n");
+		err("authmysql: critical error while "
+				 "parsing substitution variable");
 		return NULL;
 	}
 	if (len < 1)
 	{
-		fprintf (stderr, "authmysql: unknown empty substitution "
-				 "variable - aborting\n");
+		err("authmysql: unknown empty substitution "
+				 "variable - aborting");
 		return NULL;
 	}
 	if (len > MAX_SUBSTITUTION_LEN)
 	{
-		fprintf (stderr, "authmysql: variable name too long "
-				 "while parsing substitution\n"
-				 "authmysql: name begins with "
+		err("authmysql: variable name too long "
+				 "while parsing substitution. "
+				 "name begins with "
 				 SV_BEGIN_MARK
-				 "%.*s...\n", MAX_SUBSTITUTION_LEN, begin);
+				 "%.*s...", MAX_SUBSTITUTION_LEN, begin);
 		return NULL;
 	}
 	
@@ -264,11 +294,11 @@ struct var_data *vdp;
 			return vdp;
 		}
 	
-	fprintf (stderr, "authmysql: unknown substitution variable "
+	err("authmysql: unknown substitution variable "
 			 SV_BEGIN_MARK
 			 "%.*s"
 			 SV_END_MARK
-			 "\n", len, begin);
+			 , (int)len, begin);
 	
 	return NULL;
 }
@@ -278,8 +308,8 @@ static int ParsePlugin_counter (const char *p, size_t length, void *vp)
 {
 	if (!p || !vp || length < 0)
 	{
-		fprintf (stderr, "authmysql: bad arguments while counting "
-				 "query string\n");
+		err("authmysql: bad arguments while counting "
+				 "query string");
 		return -1;
 	}
 	
@@ -295,8 +325,8 @@ char	**strptr = (char **) vp;
 
 	if (!p || !vp || length < 0)
 	{
-		fprintf (stderr, "authmysql: bad arguments while building "
-				 "query string\n");
+		err("authmysql: bad arguments while building "
+				 "query string");
 		return -1;
 	}
 	
@@ -322,14 +352,14 @@ struct var_data	*v_ptr;
 		source = "";
 	if (!result)
 	{
-		fprintf (stderr, "authmysql: no memory allocated for result"
-				 "while parser core was invoked\n");
+		err("authmysql: no memory allocated for result "
+				 "while parser core was invoked");
 		return -1;
 	}
 	if (!vdt)
 	{
-		fprintf (stderr, "authmysql: no substitution table found "
-				 "while parser core was invoked\n");
+		err("authmysql: no substitution table found "
+				 "while parser core was invoked");
 		return -1;
 	}
 	
@@ -339,11 +369,11 @@ struct var_data	*v_ptr;
 		e = strstr (p, SV_END_MARK);
 		if (!e)
 		{
-			fprintf (stderr, "authmysql: syntax error in "
+			err("authmysql: syntax error in "
 					 "substitution "
-					 "- no closing symbol fould!\n"
-					 "authmysql: bad variable begins with:"
-					 "%.*s...\n", MAX_SUBSTITUTION_LEN, p);
+					 "- no closing symbol found! "
+					 "bad variable begins with:"
+					 "%.*s...", MAX_SUBSTITUTION_LEN, p);
 			return -1;
 		}
 		
@@ -396,8 +426,8 @@ size_t	buf_size	= 2;
 	if (source == NULL || *source == '\0' || 
 	    vdt == NULL    || vdt[0].name == NULL)
 	{
-		fprintf (stderr, "authmysql: source clause is empty "
-				 "- this is critical error\n");
+		err("authmysql: source clause is empty "
+		       "- this is critical error");
 		return NULL;
 	}
 
@@ -503,36 +533,46 @@ char		*q;
 }
 
 /* siefca@pld.org.pl */
-static const char *validate_password (const char *password)
+
+static const char *validateMyPassword (const char *password)
 {
-static char	pass_buf[260];
+static char pass_buf[2][540]; /* Use two buffers, see parse_chpass_clause */
+static int next_pass=0;
 const char	*p;
-char		*q;
+char		*q, *endq;
 	
 	if (!password || *password == '\0' || (strlen(password)) > 256)
 		return NULL;
 	
+	next_pass= 1-next_pass;
+
 	p = password;
-	q = pass_buf;
+	q = pass_buf[next_pass];
+	endq = q + sizeof pass_buf[next_pass];
 	
-	while (*p)
+	while (*p && q < endq)
+	{
 		if (*p == '\"' || *p == '\\' || *p == '\'')
 			*q++ = '\\';
-		else
-			*q++ = *p++;	
+		*q++ = *p++;
+	}
+	
+	if (q >= endq)
+		return NULL;
 	
 	*q = '\0';
-	return pass_buf;
+	return pass_buf[next_pass];
 }
 
-
 /* siefca@pld.org.pl */
 static char *parse_select_clause (const char *clause, const char *username,
-				  const char *defdomain)
+				  const char *defdomain,
+				  const char *service)
 {
 static struct var_data vd[]={
 	    {"local_part",	NULL,	sizeof("local_part"),	0},
 	    {"domain",		NULL,	sizeof("domain"),	0},
+	    {"service",		NULL,	sizeof("service"),	0},
 	    {NULL,		NULL,	0,			0}};
 
 	if (clause == NULL || *clause == '\0' ||
@@ -543,7 +583,8 @@ static struct var_data vd[]={
 	vd[1].value	= get_domain (username, defdomain);
 	if (!vd[0].value || !vd[1].value)
 		return NULL;
-	
+	vd[2].value     = service;
+
 	return (parse_string (clause, vd));
 }
 
@@ -566,8 +607,8 @@ static struct var_data vd[]={
 
 	vd[0].value	= get_localpart (username);
 	vd[1].value	= get_domain (username, defdomain);
-	vd[2].value	= validate_password (newpass);
-	vd[3].value	= validate_password (newpass_crypt);
+	vd[2].value	= validateMyPassword (newpass);
+	vd[3].value	= validateMyPassword (newpass_crypt);
 	
 	if (!vd[0].value || !vd[1].value ||
 	    !vd[2].value || !vd[3].value)	return NULL;
@@ -575,46 +616,36 @@ static struct var_data vd[]={
 	return (parse_string (clause, vd));
 }
 
-struct authmysqluserinfo *auth_mysql_getuserinfo(const char *username)
+struct authmysqluserinfo *auth_mysql_getuserinfo(const char *username,
+						 const char *service)
 {
 const char *user_table	=NULL;
 const char *defdomain	=NULL;
 char	*querybuf, *p;
 MYSQL_ROW	row;
 MYSQL_RES	*result;
+int		num_fields;
+char		*endp;
 
 const char  *crypt_field	=NULL, 
-	    *clear_field	=NULL,
-	    *maildir_field	=NULL,
-	    *home_field		=NULL,
-	    *name_field		=NULL,
-	    *login_field	=NULL,
-	    *uid_field		=NULL,
-	    *gid_field		=NULL,
-	    *quota_field	=NULL,
-	    *where_clause	=NULL,
-	    *select_clause	=NULL; /* siefca@pld.org.pl */
+	*clear_field	=NULL,
+	*maildir_field	=NULL,
+	*home_field		=NULL,
+	*name_field		=NULL,
+	*login_field	=NULL,
+	*uid_field		=NULL,
+	*gid_field		=NULL,
+	*quota_field	=NULL,
+	*options_field	=NULL,
+	*where_clause	=NULL,
+	*select_clause	=NULL; /* siefca@pld.org.pl */
 
 static const char query[]=
-	"SELECT %s, %s, %s, %s, %s, %s, %s, %s, %s FROM %s WHERE %s = \"";
+	"SELECT %s, %s, %s, %s, %s, %s, %s, %s, %s, %s FROM %s WHERE %s = \"";
 
 	if (do_connect())	return (0);
 
-	if (ui.username)
-		free(ui.username);
-	if (ui.cryptpw)
-		free(ui.cryptpw);
-	if (ui.clearpw)
-		free(ui.clearpw);
-	if (ui.home)
-		free(ui.home);
-	if (ui.maildir)
-		free(ui.maildir);
-	if (ui.quota)
-		free(ui.quota);
-	if (ui.fullname)
-		free(ui.fullname);
-	memset(&ui, 0, sizeof(ui));
+	initui();
 
 	select_clause=read_env("MYSQL_SELECT_CLAUSE");
 	defdomain=read_env("DEFAULT_DOMAIN");	
@@ -625,8 +656,8 @@ static const char query[]=
 
 		if (!user_table)
 		{
-			fprintf(stderr, "authmysql: MYSQL_USER_TABLE not set in "
-				AUTHMYSQLRC ".\n");
+			err("authmysql: MYSQL_USER_TABLE not set in "
+				AUTHMYSQLRC ".");
 			return (0);
 		}
 
@@ -636,9 +667,8 @@ static const char query[]=
 
 		if (!crypt_field && !clear_field)
 		{
-			fprintf(stderr,
-				"authmysql: MYSQL_CRYPT_PWFIELD and "
-				"MYSQL_CLEAR_PWFIELD not set in " AUTHMYSQLRC ".\n");
+			err("authmysql: MYSQL_CRYPT_PWFIELD and "
+				"MYSQL_CLEAR_PWFIELD not set in " AUTHMYSQLRC ".");
 			return (0);
 		}
 		if (!crypt_field) crypt_field="\"\"";
@@ -657,12 +687,17 @@ static const char query[]=
 		home_field = read_env("MYSQL_HOME_FIELD");
 		if (!home_field) home_field = "home";
 
-		maildir_field=read_env("MYSQL_MAILDIR_FIELD");
+		maildir_field=read_env(service && strcmp(service, "courier")==0
+				       ? "MYSQL_DEFAULTDELIVERY"
+				       : "MYSQL_MAILDIR_FIELD");
 		if (!maildir_field) maildir_field="\"\"";
 
 		quota_field=read_env("MYSQL_QUOTA_FIELD");
 		if (!quota_field) quota_field="\"\""; 
 
+		options_field=read_env("MYSQL_AUXOPTIONS_FIELD");
+		if (!options_field) options_field="\"\"";
+
 		where_clause=read_env("MYSQL_WHERE_CLAUSE");
 		if (!where_clause) where_clause = "";
 	}
@@ -675,7 +710,9 @@ static const char query[]=
 			+ strlen(crypt_field) + strlen(clear_field) + strlen(maildir_field)
 			+ strlen(uid_field) + strlen(gid_field) + 2 * strlen(login_field)
 			+ strlen(home_field) + strlen(quota_field) + strlen(where_clause)
-			+ strlen(name_field));
+				+strlen(options_field)
+			+ strlen(name_field) + strlen(username)
+				+ strlen(defdomain ? defdomain:""));
 
 		if (!querybuf)
 		{
@@ -685,7 +722,7 @@ static const char query[]=
 
 		sprintf(querybuf, query, login_field, crypt_field, clear_field, 
 			uid_field, gid_field, home_field, maildir_field, quota_field,
-			name_field, user_table, login_field);
+			name_field, options_field, user_table, login_field);
 
 		p=querybuf+strlen(querybuf);
 
@@ -701,14 +738,17 @@ static const char query[]=
 	else
 	{
 		/* siefca@pld.org.pl */
-		querybuf=parse_select_clause (select_clause, username, defdomain);
+		querybuf=parse_select_clause (select_clause, username,
+					      defdomain, service);
 		if (!querybuf) return 0;
 	}
 
+	dprintf("SQL query: %s", querybuf);
 	if (mysql_query (mysql, querybuf))
 	{
 		/* <o.blasnik@nextra.de> */
 
+		dprintf("mysql_query failed, reconnecting");
 		auth_mysql_cleanup();
 
 		if (do_connect())
@@ -719,6 +759,7 @@ static const char query[]=
 
 		if (mysql_query (mysql, querybuf))
 		{
+			dprintf("mysql_query failed second time, giving up");
 			free(querybuf);
 			auth_mysql_cleanup();
 			/* Server went down, that's OK,
@@ -735,37 +776,70 @@ static const char query[]=
 		if (mysql_num_rows(result))
 		{
 			row = mysql_fetch_row (result);
-			ui.username=strdup(row[0]);
-			ui.cryptpw=strdup(row[1]);
-			ui.clearpw=strdup(row[2]);
-			ui.uid=atol(row[3]);
-			ui.gid=atol(row[4]);
-			ui.home=strdup(row[5]);
-			ui.maildir=strdup(row[6]);
-			ui.quota=strdup(row[7]);
-			ui.fullname=strdup(row[8]);
-			if (!ui.username || !ui.cryptpw ||
-				!ui.home || !ui.maildir || 
-				!ui.quota)
+			num_fields = mysql_num_fields (result);
+
+			if (num_fields < 6)
 			{
+				dprintf("incomplete row, only %d fields returned",
+					num_fields);
 				mysql_free_result(result);
-				return (0);
+				return(0);
 			}
 
-			if (!ui.cryptpw[0])
+			if (row[0] && row[0][0])
+				ui.username=strdup(row[0]);
+			if (row[1] && row[1][0])
+				ui.cryptpw=strdup(row[1]);
+			if (row[2] && row[2][0])
+				ui.clearpw=strdup(row[2]);
+			/* perhaps authmysql needs a glob_uid/glob_gid feature
+			   like authldap? */
+			if (!row[3] || !row[3][0] ||
+			   (ui.uid=strtol(row[3], &endp, 10), endp[0] != '\0'))
 			{
-				free(ui.cryptpw);
-				ui.cryptpw=0;
+				dprintf("invalid value for uid: '%s'",
+					row[3] ? row[3] : "<null>");
+				mysql_free_result(result);
+				return 0;
 			}
-
-			if (!ui.clearpw[0])
+			if (!row[4] || !row[4][0] ||
+			   (ui.gid=strtol(row[4], &endp, 10), endp[0] != '\0'))
 			{
-				free(ui.clearpw);
-				ui.clearpw=0;
+				dprintf("invalid value for gid: '%s'",
+					row[4] ? row[4] : "<null>");
+				mysql_free_result(result);
+				return 0;
 			}
+			if (row[5] && row[5][0])
+				ui.home=strdup(row[5]);
+			else
+			{
+				dprintf("required value for 'home' (column 6) is missing");
+				mysql_free_result(result);
+				return(0);
+			}
+			if (num_fields > 6 && row[6] && row[6][0])
+				ui.maildir=strdup(row[6]);
+			if (num_fields > 7 && row[7] && row[7][0])
+				ui.quota=strdup(row[7]);
+			if (num_fields > 8 && row[8] && row[8][0])
+				ui.fullname=strdup(row[8]);
+			if (num_fields > 9 && row[9] && row[9][0])
+				ui.options=strdup(row[9]);
+		}
+		else
+		{
+			dprintf("zero rows returned");
+			mysql_free_result(result);
+			return (&ui); /* User not found */
 		}
-		mysql_free_result(result);
 	}
+	else
+	{
+		dprintf("mysql_store_result failed");
+		return (0);
+	}
+	mysql_free_result(result);
 	return (&ui);
 }
 
@@ -827,7 +901,7 @@ int auth_mysql_setpass(const char *user, const char *pass)
 		sql_buf=malloc(strlen(crypt_field ? crypt_field:"")
 				+ strlen(clear_field ? clear_field:"")
 				+ strlen(defdomain ? defdomain:"")
-				+ strlen(login_field) + l + strlen(newpass_crypt)
+				+ strlen(login_field) + l + strlen(newpass_crypt_ptr)
 				+ strlen(user_table)
 				+ strlen(where_clause ? where_clause:"")
 				+ 200);
@@ -838,7 +912,7 @@ int auth_mysql_setpass(const char *user, const char *pass)
 					    user,
 					    defdomain,
 					    pass,
-					    newpass_crypt);
+					    newpass_crypt_ptr);
 	}
 	
 
@@ -900,11 +974,175 @@ int auth_mysql_setpass(const char *user, const char *pass)
 		
 	} /* end of: if (!chpass_clause) */
 
+	if (auth_debug_login_level >= 2)
+		dprintf("setpass SQL: %s", sql_buf);
 	if (mysql_query (mysql, sql_buf))
 	{
+		dprintf("setpass SQL failed");
 		rc= -1;
 		auth_mysql_cleanup();
 	}
 	free(sql_buf);
 	return (rc);
 }
+
+void auth_mysql_enumerate( void(*cb_func)(const char *name,
+					  uid_t uid,
+					  gid_t gid,
+					  const char *homedir,
+					  const char *maildir,
+					  void *void_arg),
+			   void *void_arg)
+{
+	const char *user_table	=NULL;
+	const char *defdomain	=NULL;
+	char	*querybuf, *p;
+	MYSQL_ROW	row;
+	MYSQL_RES	*result;
+
+	const char  *maildir_field	=NULL,
+		*home_field		=NULL,
+		*login_field	=NULL,
+		*uid_field		=NULL,
+		*gid_field		=NULL,
+		*where_clause	=NULL,
+		*select_clause	=NULL; /* siefca@pld.org.pl */
+
+	static const char query[]=
+		"SELECT %s, %s, %s, %s, %s FROM %s WHERE 1=1";
+
+	if (do_connect())	return;
+
+	initui();
+
+	select_clause=read_env("MYSQL_ENUMERATE_CLAUSE");
+	defdomain=read_env("DEFAULT_DOMAIN");	
+	
+	if (!select_clause)
+	{
+		user_table=read_env("MYSQL_USER_TABLE");
+
+		if (!user_table)
+		{
+			err("authmysql: MYSQL_USER_TABLE not set in "
+				AUTHMYSQLRC ".");
+			return;
+		}
+
+		uid_field = read_env("MYSQL_UID_FIELD");
+		if (!uid_field) uid_field = "uid";
+
+		gid_field = read_env("MYSQL_GID_FIELD");
+		if (!gid_field) gid_field = "gid";
+
+		login_field = read_env("MYSQL_LOGIN_FIELD");
+		if (!login_field) login_field = "id";
+
+		home_field = read_env("MYSQL_HOME_FIELD");
+		if (!home_field) home_field = "home";
+
+		maildir_field=read_env("MYSQL_MAILDIR_FIELD");
+		if (!maildir_field) maildir_field="\"\"";
+
+		where_clause=read_env("MYSQL_WHERE_CLAUSE");
+		if (!where_clause) where_clause = "";
+	}
+	
+	if (!defdomain)	defdomain="";
+
+	if (!select_clause) /* siefca@pld.org.pl */
+	{
+		querybuf=malloc(sizeof(query) + 100 + strlen(user_table) + strlen(defdomain)
+				+ strlen(maildir_field)
+				+ strlen(uid_field) + strlen(gid_field)
+				+ strlen(login_field)
+				+ strlen(home_field) + strlen(where_clause)
+				+ strlen(defdomain ? defdomain:""));
+
+		if (!querybuf)
+		{
+			perror("malloc");
+			return;
+		}
+
+		sprintf(querybuf, query, login_field, 
+			uid_field, gid_field, home_field, maildir_field,
+			user_table);
+
+		p=querybuf+strlen(querybuf);
+	
+		if (strcmp(where_clause, "")) {
+			strcat(p, " AND (");
+			strcat(p, where_clause);
+			strcat(p, ")");
+		}
+	}
+	else
+	{
+		/* siefca@pld.org.pl */
+		querybuf=parse_select_clause (select_clause, "*",
+					      defdomain, "enumerate");
+		if (!querybuf) return;
+	}
+
+	if (mysql_query (mysql, querybuf))
+	{
+		/* <o.blasnik@nextra.de> */
+
+		auth_mysql_cleanup();
+
+		if (do_connect())
+		{
+			free(querybuf);
+			return;
+		}
+
+		if (mysql_query (mysql, querybuf))
+		{
+			free(querybuf);
+			auth_mysql_cleanup();
+			/* Server went down, that's OK,
+			** try again next time.
+			*/
+			return;
+		}
+	}
+	free(querybuf);
+
+	result = mysql_store_result (mysql);       
+	if (result)
+	{
+		if (mysql_num_rows(result))
+		{
+			const char *username;
+			uid_t uid;
+			gid_t gid;
+			const char *homedir;
+			const char *maildir;
+
+			while ((row = mysql_fetch_row (result)) != NULL)
+			{
+				if(!row[0] || !row[1] || !row[2] ||
+				   !row[3])
+				{
+					continue;
+				}
+
+				username=row[0];
+
+				uid=atol(row[1]);
+				gid=atol(row[2]);
+				homedir=row[3];
+				maildir=row[4];
+
+				if (maildir && !*maildir)
+					maildir=NULL;
+
+				(*cb_func)(username, uid, gid, homedir,
+					   maildir, void_arg);
+			}
+			(*cb_func)(NULL, 0, 0, NULL, NULL, void_arg);
+		}
+		mysql_free_result(result);
+	}
+}
