@@ -18,11 +18,13 @@ struct authpgsqluserinfo {
 	char *home;
 	char *maildir;
 	char *quota;
+	char *options;
 	uid_t uid;
 	gid_t gid;
 	} ;
 
-extern struct authpgsqluserinfo *auth_pgsql_getuserinfo(const char *);
+extern struct authpgsqluserinfo *auth_pgsql_getuserinfo(const char *,
+							const char *service);
 extern void auth_pgsql_cleanup();
 
 extern int auth_pgsql_setpass(const char *, const char *);
