@@ -14,8 +14,9 @@
 #include	<crypt.h>
 #endif
 #include	"auth.h"
+#include	"debug.h"
 
-static const char rcsid[]="$Id: checkpassword.c,v 1.9 2001/06/20 03:33:02 mrsam Exp $";
+static const char rcsid[]="$Id: checkpassword.c,v 1.10 2004/04/18 15:54:39 mrsam Exp $";
 
 #if HAVE_CRYPT
 #if NEED_CRYPT_PROTOTYPE
@@ -31,7 +32,7 @@ extern int authcheckpasswordmd5(const char *, const char *);
 extern int authcheckpasswordsha1(const char *, const char *);
 #endif
 
-int authcheckpassword(const char *password, const char *encrypted_password)
+static int do_authcheckpassword(const char *password, const char *encrypted_password)
 {
 #if	HAVE_MD5LIB
 	if (strncmp(encrypted_password, "$1$", 3) == 0
@@ -55,3 +56,18 @@ int authcheckpassword(const char *password, const char *encrypted_password)
 #endif
 				);
 }
+
+int authcheckpassword(const char *password, const char *encrypted_password)
+{
+int rc;
+
+	rc=do_authcheckpassword(password, encrypted_password);
+	if (rc == 0)
+		dprintf("password matches successfully");
+	else if (auth_debug_login_level >= 2)
+		dprintf("supplied password '%s' does not match encrypted password '%s'",
+			password, encrypted_password);
+	else
+		dprintf("supplied password does not match encrypted password");
+	return rc;
+}
