@@ -12,12 +12,13 @@
 #include	<errno.h>
 #include	<pwd.h>
 #include	"auth.h"
+#include	"debug.h"
 
 #if	HAVE_SHADOW_H
 #include	<shadow.h>
 #endif
 
-static const char rcsid[]="$Id: preauthpam.c,v 1.8 2001/12/22 21:25:13 mrsam Exp $";
+static const char rcsid[]="$Id: preauthpam.c,v 1.10 2004/05/09 02:52:23 mrsam Exp $";
 
 int auth_pam_pre(const char *userid, const char *service,
         int (*callback)(struct authinfo *, void *),
@@ -34,6 +35,7 @@ struct spwd *spw;
 	if ((pw=getpwnam(userid)) == 0)
 	{
 		if (errno == ENOMEM)	return (1);
+		dprintf("authpam: username '%s' not found in password file", userid);
 		errno=EPERM;
 		return (-1);
 	}
@@ -49,6 +51,7 @@ struct spwd *spw;
 	if ((spw=getspnam(userid)) != 0)
 		auth.passwd=spw->sp_pwdp;
 #endif
+	auth_debug_authinfo("DEBUG: authpam: ", &auth, 0, pw->pw_passwd);
 
 	return ((*callback)(&auth, arg));
 }
