@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 1999 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2003 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
@@ -20,7 +20,7 @@
 #include	"authpgsql.h"
 
 
-static const char rcsid[]="$Id: preauthpgsql.c,v 1.1 2001/10/06 19:01:23 mrsam Exp $";
+static const char rcsid[]="$Id: preauthpgsql.c,v 1.3 2004/01/19 19:34:09 mrsam Exp $";
 
 
 int	auth_pgsql_pre(const char *user, const char *service,
@@ -29,7 +29,7 @@ int	auth_pgsql_pre(const char *user, const char *service,
 struct authpgsqluserinfo *authinfo;
 struct	authinfo	aa;
 
-	authinfo=auth_pgsql_getuserinfo(user);
+	authinfo=auth_pgsql_getuserinfo(user, service);
 
 	if (!authinfo)		/* Fatal error - such as MySQL being down */
 		return (1);
@@ -51,5 +51,6 @@ struct	authinfo	aa;
 	aa.fullname=authinfo->fullname;
 	aa.quota=authinfo->quota && authinfo->quota[0] ?
 		authinfo->quota:0;
+	aa.options=authinfo->options;
 	return ((*callback)(&aa, arg));
 }
