@@ -16,8 +16,9 @@
 #endif
 
 #include	"auth.h"
+#include	"debug.h"
 
-static const char rcsid[]="$Id: preauthpwd.c,v 1.5 2000/02/20 21:48:36 mrsam Exp $";
+static const char rcsid[]="$Id: preauthpwd.c,v 1.7 2004/05/09 02:52:23 mrsam Exp $";
 
 int auth_pwd_pre(const char *userid, const char *service,
 	int (*callback)(struct authinfo *, void *),
@@ -41,5 +42,6 @@ struct passwd *pw;
 	auth.fullname=pw->pw_gecos;
 	auth.passwd=pw->pw_passwd;
 
+	auth_debug_authinfo("DEBUG: authpwd: ", &auth, 0, pw->pw_passwd);
 	return ((*callback)(&auth, arg));
 }
