@@ -20,8 +20,9 @@
 
 
 #include	"auth.h"
+#include	"debug.h"
 
-static const char rcsid[]="$Id: preauthshadow.c,v 1.5 2000/02/20 21:48:36 mrsam Exp $";
+static const char rcsid[]="$Id: preauthshadow.c,v 1.7 2004/05/09 02:52:23 mrsam Exp $";
 
 int auth_shadow_pre(const char *userid, const char *service,
 	int (*callback)(struct authinfo *, void *),
@@ -49,5 +50,6 @@ struct spwd *spw;
 	if ((spw=getspnam(userid)) != 0)
 		auth.passwd=spw->sp_pwdp;
 
+	auth_debug_authinfo("DEBUG: authshadow: ", &auth, 0, pw->pw_passwd);
 	return ((*callback)(&auth, arg));
 }
