@@ -1,12 +1,17 @@
 /*
-** Copyright 1998 - 2000 Double Precision, Inc.
+** Copyright 1998 - 2003 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
 /*
-** $Id: cginocache.c,v 1.3 2000/09/28 01:17:13 mrsam Exp $
+** $Id: cginocache.c,v 1.5 2003/03/23 01:35:05 mrsam Exp $
 */
+#if HAVE_LIBFCGI
+#include <stdlib.h>
+#include "fcgi_stdio.h"
+#else
 #include	<stdio.h>
+#endif
 
 void cginocache()
 {
