@@ -38,7 +38,7 @@
 #include	"numlib/numlib.h"
 #include	"tcpd/spipe.h"
 #include	"tcpd/tlsclient.h"
-static const char rcsid[]="$Id: courieresmtpd.c,v 1.44 2004/04/24 19:56:19 mrsam Exp $";
+static const char rcsid[]="$Id: courieresmtpd.c,v 1.45 2004/08/08 22:01:47 mrsam Exp $";
 
 static char helobuf[256];
 static char authuserbuf[256];
@@ -64,6 +64,14 @@ static const char *tcpremoteip, *tcpremotehost;
 
 static time_t teergrube=INIT_TEERGRUBE;
 
+static void tarpit()
+{
+	sleep(teergrube);
+	teergrube *= 2;
+	if (teergrube > MAX_TEERGRUBE)
+		teergrube=MAX_TEERGRUBE;
+}
+
 void addiovec_error(const char *q, const char *p)
 {
 const char *ident=getenv("TCPREMOTEINFO");
@@ -93,12 +101,7 @@ const char *ident=getenv("TCPREMOTEINFO");
 	iovflush();
 
 	if (*p == '5')
-	{
-		sleep(teergrube);
-		teergrube *= 2;
-		if (teergrube > MAX_TEERGRUBE)
-			teergrube=MAX_TEERGRUBE;
-	}
+		tarpit();
 }
 
 static void showbanner()
@@ -242,17 +245,8 @@ int	rc;
 	fclose(submit_to);
 	rc=submit_readrcprintcrlf();
 	(void)submit_wait();
-	if (rc)
-	{
-		if (rc == -5 )
-		{
-			sleep(teergrube);
-			teergrube *= 2;
-			if (teergrube > MAX_TEERGRUBE)
-				teergrube=MAX_TEERGRUBE;
-		}
-	}
-	else	teergrube=INIT_TEERGRUBE;
+	if (rc == 0)
+		teergrube=INIT_TEERGRUBE;
 	fclose(fromsubmit);
 }
 
@@ -592,13 +586,6 @@ int	rc;
 	if (rc)
 	{
 		iovflush();
-		if (rc == -5)
-		{
-			sleep(teergrube);
-			teergrube *= 2;
-			if (teergrube > MAX_TEERGRUBE)
-				teergrube=MAX_TEERGRUBE;
-		}
 	}
 	return (rc);
 }
@@ -769,14 +756,6 @@ int	rc;
 	if (rc)
 	{
 		iovflush();
-		if (rc == -5)
-		{
-			sleep(teergrube);
-			teergrube *= 2;
-			if (teergrube > MAX_TEERGRUBE)
-				teergrube=MAX_TEERGRUBE;
-		}
-
 	}
 	return (rc);
 }
@@ -880,17 +859,8 @@ int	rc;
 		exit(0);
 	}
 
-	if (rc)
-	{
-		if (rc == -5)
-		{
-			sleep(teergrube);
-			teergrube *= 2;
-			if (teergrube > MAX_TEERGRUBE)
-				teergrube=MAX_TEERGRUBE;
-		}
-	}
-	else	teergrube=INIT_TEERGRUBE;
+	if (rc == 0)
+		teergrube=INIT_TEERGRUBE;
 	fclose(fromsubmit);
 }
 
@@ -943,6 +913,8 @@ int	authenticated=0;
 		exit(1);
 	}
 
+	submit_set_teergrube(tarpit);
+
 	if (strcmp(argv[0], "sendmail") == 0)
 		clog_open_stderr("courieresmtpd");
 		/* We are being called by the sendmail command line wrapper */
