@@ -309,7 +309,7 @@ static int get_sourceaddr(int af, RFC1035_NETADDR *addrbuf,
 
 static void sendesmtp(struct moduledel *del, struct ctlfile *ctf)
 {
-char *smtproute;
+	char *smtproute;
 
 	if (!host)
 		host=strcpy(courier_malloc(strlen(del->host)+1), del->host);
@@ -807,9 +807,9 @@ time_t	current_time;
 
 	if ( sox_select(sockbuf.fd+1, &fdr, &fdw, 0, &tv) > 0)
 	{
-		if (FD_ISSET(sockbuf.fd, &fdw))
+		if (waitw && FD_ISSET(sockbuf.fd, &fdw))
 			*waitw=1;
-		if (FD_ISSET(sockbuf.fd, &fdr))
+		if (waitr && FD_ISSET(sockbuf.fd, &fdr))
 			*waitr=1;
 		return;
 	}
