@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2002 Double Precision, Inc.
+** Copyright 1998 - 2003 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -52,8 +52,9 @@
 #include	"maildir/maildirquota.h"
 #include	"liblock/config.h"
 #include	"liblock/liblock.h"
+#include	"liblock/mail.h"
 
-static const char rcsid[]="$Id: deliver.c,v 1.12 2002/02/23 21:21:22 mrsam Exp $";
+static const char rcsid[]="$Id: deliver.c,v 1.21 2003/01/23 13:26:51 mrsam Exp $";
 
 extern char *local_dotcourier(const char *, const char *, const char **);
 extern char *local_extension();
@@ -433,11 +434,11 @@ static int savemessage(const char *extension, const char *sender,
 {
 FILE	*delivf;
 char	buf[BUFSIZ];
-char	*p, *q;
 int	c;
 static unsigned counter=0;
-int	quota_fd;
 struct	stat stat_buf;
+struct maildirsize quotainfo;
+struct maildir_tmpcreate_info createInfo;
 
 	umask(077);
 
@@ -445,15 +446,40 @@ struct	stat stat_buf;
 	{
 		/* Ok, perhaps this is a mailbox */
 
-		if ( ll_lock_ex(fileno(delivf)) ||
-			fseek(delivf, 0L, SEEK_END) == -1 ||
-			(mbox_size=ftell(delivf)) == -1)
+		struct ll_mail *ll=ll_mail_alloc(name);
+
+		fclose(delivf);
+
+		if (!ll)
 		{
-			if (delivf)	fclose(delivf);
 			delivery_error(name);	/* I Give up */
 		}
 
-		mbox_fd=fileno(delivf);
+		while ((mbox_fd=ll_mail_open(ll)) < 0)
+		{
+			switch (errno) {
+			case EEXIST:
+			case EAGAIN:
+				sleep(5);
+				continue;
+				break;
+			}
+
+			delivery_error(name);
+		}
+
+		delivf=fdopen(mbox_fd, "w");
+
+		if (delivf == NULL || fseek(delivf, 0L, SEEK_END) < 0
+		    || (mbox_size=ftell(delivf)) == (off_t)-1)
+		{
+			if (delivf)
+				fclose(delivf);
+			close(mbox_fd);
+			ll_mail_free(ll);
+			delivery_error(name);	/* I Give up */
+		}
+
 		signal(SIGHUP, truncmbox);
 		signal(SIGINT, truncmbox);
 		signal(SIGQUIT, truncmbox);
@@ -497,8 +523,12 @@ struct	stat stat_buf;
 #if	HAVE_FTRUNCATE
 			ftruncate(mbox_fd, mbox_size);
 #endif
+			close(mbox_fd);
+			ll_mail_free(ll);
 			delivery_error("mailbox.close");
 		}
+		close(mbox_fd);
+		ll_mail_free(ll);
 		return (0);
 	}
 
@@ -510,55 +540,83 @@ struct	stat stat_buf;
 
 	stat_buf.st_size += strlen(dtline) + strlen(rpline) + 2;
 
-	if (quota && *quota && maildir_checkquota(name, &quota_fd, quota,
-		0, 1) && errno != EAGAIN)
+	if (maildir_quota_add_start(name, &quotainfo, stat_buf.st_size, 1,
+				    quota && *quota != 0 ? quota:NULL))
 	{
-		if (quota_fd >= 0)
-			close(quota_fd);
 		errno=ENOSPC;
 		delivery_error("out of memory.");
 	}
 
+	maildir_closequotafile(&quotainfo);	// For now.
+
 	sprintf(buf, "%u", ++counter);
 
-	while (maildir_try_create_hostname(name, buf,
-		stat_buf.st_size, config_me(), &p, &q))
-	{
-		sleep(3);
-	}
+	maildir_tmpcreate_init(&createInfo);
+	createInfo.maildir=name;
+	createInfo.uniq=buf;
+	createInfo.msgsize=stat_buf.st_size;
+	createInfo.doordie=1;
 
-	if ((delivf=fopen(p, "w")) == 0)
+	if ((delivf=maildir_tmpcreate_fp(&createInfo)) == NULL)
 	{
-		delivery_error("maildir.open");
+		snprintf(buf, BUFSIZ-1,
+			 "maildir.open: %s: %s", name,
+			 strerror(errno));
+		buf[BUFSIZ-1] = 0;
+		delivery_error(buf);
 		return (-1);
 	}
 
 	fprintf(delivf, "%s\n%s\n", dtline, rpline);
 
-	while ((c=getc(f)) >= 0)
-		putc(c, delivf);
+	{
+		char buffer[BUFSIZ];
+		int n;
+
+		while ((n=fread(buffer, 1, sizeof(buffer), f)) > 0)
+			if (fwrite(buffer, 1, n, delivf)  != n)
+				break;
+	}
 
 	if ( ferror(f) || fflush(delivf) || ferror(delivf)
 #if	EXPLICITSYNC
 			|| fsync(fileno(delivf))
 #endif
 			|| fclose(delivf)
-			|| (delivf=0, rename(p, q)))
+			|| (delivf=0, maildir_movetmpnew(createInfo.tmpname,
+							 createInfo.newname)))
 	{
+		snprintf(buf, BUFSIZ-1,
+			 "maildir.close: %s: %s", name,
+			 strerror(errno));
+		buf[BUFSIZ-1] = 0;
+
 		if (delivf)	fclose(delivf);
-		unlink(p);
-		delivery_error("maildir.close");
+		unlink(createInfo.newname);
+		delivery_error(buf);
 		return (-1);
 	}
-	free(p);
-	free(q);
 
-	if (quota && *quota)
+#if EXPLICITDIRSYNC
+
 	{
-		maildir_addquota(name, quota_fd, quota, stat_buf.st_size, 1);
-		if (quota_fd >= 0)
-			close (quota_fd);
+		int fd;
+
+		strcpy(strrchr(createInfo.newname, '/')+1, ".");
+
+		fd=open(createInfo.newname, O_RDONLY);
+
+		if (fd >= 0)
+		{
+			fsync(fd);
+			close(fd);
+		}
 	}
+#endif
+
+	maildir_tmpcreate_free(&createInfo);
+
+	maildir_quota_deleted(name, stat_buf.st_size, 1);
 	return (0);
 }
 
