@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2001 Double Precision, Inc.
+** Copyright 1998 - 2004 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -36,7 +36,7 @@
 
 #define DEFAULTNAME	"alias@"
 
-static const char rcsid[]="$Id: local.c,v 1.19 2001/10/21 18:09:41 mrsam Exp $";
+static const char rcsid[]="$Id: local.c,v 1.21 2004/05/09 17:47:04 mrsam Exp $";
 
 extern char *local_dotcourier(const char *, const char *, const char **);
 
@@ -136,11 +136,14 @@ struct rfc822token *p, **prevp;
 char	*addr, *ext;
 int	i;
 struct localauthinfo lai;
-char	*hostdomain;
+char	*hostdomain=NULL;
 struct	rfc822token	hostdomaint;
 #if LOCAL_EXTENSIONS
 char *atdomain;
 #endif
+const char *authmodules=getenv("AUTHMODULES");
+
+
 	if (!cleanup_set)
 	{
 		atexit(cleanup);
@@ -195,10 +198,11 @@ char *atdomain;
 	if (!addr)	clog_msg_errno();
 	locallower(addr);
 
-	if (strchr(addr, '!'))
+	if (strchr(addr, '!') || *addr == 0)
 	{
 		/* Can't have !s in an address, that's used a delimiter
-		** for the returned data
+		** for the returned data.
+		** Can't have NULL addresses either.
 		*/
 
 		goto not_found;
@@ -267,8 +271,9 @@ char *atdomain;
 		lai.rwi=rwi;
 		lai.found=0;
 
-		i=authstaticlist_search(search_addr, "courier", MODULELIST,
-			&local_callback, &lai);
+		i=authstaticlist_search_string(search_addr, "courier",
+					       authmodules,
+					       &local_callback, &lai);
 
 #if LOCAL_EXTENSIONS
 		if (alloc_buf)
@@ -405,7 +410,9 @@ not_found:
 	*/
 
 	if ((rwi->mode & (RW_SUBMIT|RW_EXPN|RW_VERIFY)) == RW_SUBMIT &&
-		rwi->smodule && strcmp(rwi->smodule, NAME) == 0)
+	    rwi->smodule &&
+	    (strcmp(rwi->smodule, NAME) == 0 ||
+	     strcmp(rwi->smodule, "uucp") == 0))
 	{
 		(*delfunc)(rwi, rwi->ptr, rwi->ptr);
 		return;
