@@ -2,8 +2,9 @@
 #
 # TITLE: LDAP
 #
+# $Id: admin-15ldap.pl,v 1.7 2003/07/06 16:16:37 mrsam Exp $
 #
-# Copyright 2001-2004 Double Precision, Inc.  See COPYING for
+# Copyright 2001 Double Precision, Inc.  See COPYING for
 # distribution information.
 
 use webadmin;
@@ -11,26 +12,26 @@ use webadmin;
 my $errstr="";
 
 display_form("notsupp.html")
-    unless -f "$authldaprc";
+    unless -f "$libexecdir/authlib/authldap" ||
+    -f "$libexecdir/authlib/authdaemond.ldap";
 
 if ($cgi->param("Save"))
 {
     my $ldap=ReadKWConfigFile("authldaprc");
 
     my $newkw={
-	"LOCATION" => "LDAP_URI\t" . param('URI'),
+	"LOCATION" => "LDAP_SERVER\t" . param('SERVER') . "\nLDAP_PORT\t" . param('PORT'),
 	"LDAP_BASEDN" => "LDAP_BASEDN\t" . param('BASEDN'),
 	"LDAP_BINDDN" => "LDAP_BINDDN\t" . param('BINDDN') . "\nLDAP_BINDPW\t" . $$ldap{'LDAP_BINDPW'},
 	"LDAP_TIMEOUT" => "LDAP_TIMEOUT\t" . param('TIMEOUT'),
 	"LDAP_MAIL" => "LDAP_MAIL\t" . param('MAIL'),
 	"LDAP_HOMEDIR" => "LDAP_HOMEDIR\t" . param('HOMEDIR'),
-	"LDAP_PW" => "LDAP_CLEARPW\t" . param('CLEARPW') . "\nLDAP_CRYPTPW\t" . param('CRYPTPW'),
-	"LDAP_FILTER" => "LDAP_FILTER\t" . param('FILTER'),
+	"LDAP_PW" => "LDAP_CLEARPW\t" . param('CLEARPW') . "\nLDAP_CRYPTPW\t" . param('CRYPTPW')
 	} ;
 
     $errstr="\@REQUIRED\@"
 	unless required
-		 ('URI', 'BASEDN', 'TIMEOUT', 'MAIL', 'HOMEDIR') &&
+		 ('SERVER', 'PORT', 'BASEDN', 'TIMEOUT', 'MAIL', 'HOMEDIR') &&
 		     (param('CLEARPW') || param('CRYPTPW')) &&
 			 (param('UID') || param('GLOB_UID')) &&
 			     (param('GID') || param('GLOB_GID'));
@@ -69,7 +70,7 @@ if ($cgi->param("Save"))
     {
 	$errstr="\@SAVED\@";
     }
-    changed("$authdaemond restart");
+    changed("test ! -x $libexecdir/authlib/authdaemond || $libexecdir/authlib/authdaemond restart");
 }
 
 my $ldap=ReadKWConfigFile("authldaprc");
@@ -78,7 +79,7 @@ display_form("admin-15ldap.html",
 	     {
 		 "ERROR" => $errstr,
 
-		 "URI" => $$ldap{'LDAP_URI'},
+		 "SERVER" => $$ldap{'LDAP_SERVER'},
 		 "PORT" => $$ldap{'LDAP_PORT'},
 		 "BASEDN" => $$ldap{'LDAP_BASEDN'},
 
@@ -100,6 +101,5 @@ display_form("admin-15ldap.html",
 		 "CRYPTPW" => $$ldap{'LDAP_CRYPTPW'},
 		 "UID" => $$ldap{'LDAP_UID'},
 		 "GID" => $$ldap{'LDAP_GID'},
-		 "FILTER" => $$ldap{'LDAP_FILTER'},
 	     }
 	     );
