@@ -2,8 +2,9 @@
 #
 # TITLE: LDAP-based aliasing/routing
 #
+# $Id: admin-15ldapa.pl,v 1.4 2003/07/06 16:16:37 mrsam Exp $
 #
-# Copyright 2001-2004 Double Precision, Inc.  See COPYING for
+# Copyright 2001 Double Precision, Inc.  See COPYING for
 # distribution information.
 
 use webadmin;
@@ -11,7 +12,8 @@ use webadmin;
 my $errstr="";
 
 display_form("notsupp.html")
-    unless -f "$authldaprc";
+    unless -f "$libexecdir/authlib/authldap" ||
+    -f "$libexecdir/authlib/authdaemond.ldap";
 
 if ($cgi->param("Save"))
 {
@@ -19,7 +21,7 @@ if ($cgi->param("Save"))
 
     my $newkw={
 	"LDAP_ALIAS" => "LDAP_ALIAS\t" . (param('ENABLED') ? "1":"0"),
-	"LDAP_LOCATION" => "LDAP_URI\t" . param('URI'),
+	"LDAP_LOCATION" => "LDAP_SERVER\t" . param('SERVER') . "\nLDAP_PORT\t" . param('PORT'),
 	"LDAP_NUMPROCS" => "LDAP_NUMPROCS\t" . param('NUMPROCS'),
 	
 	"LDAP_BASEDN" => "LDAP_BASEDN\t" . param('BASEDN'),
@@ -44,7 +46,8 @@ display_form("admin-15ldapa.html",
 		 "ERROR" => $errstr,
 		 "ENABLED" => ("<input type=\"checkbox\" name=\"ENABLED\"" . ($$ldap{'LDAP_ALIAS'} ? " checked=\"checked\"":"")) . " />",
 
-		 "URI" => $$ldap{'LDAP_URI'},
+		 "SERVER" => $$ldap{'LDAP_SERVER'},
+		 "PORT" => $$ldap{'LDAP_PORT'},
 		 "BASEDN" => $$ldap{'LDAP_BASEDN'},
 
 		 "BINDDN" => $$ldap{'LDAP_BINDDN'},
