@@ -2,8 +2,9 @@
 #
 # TITLE: MySQL
 #
+# $Id: admin-15mysql.pl,v 1.5 2003/06/17 15:44:34 mrsam Exp $
 #
-# Copyright 2001-2004 Double Precision, Inc.  See COPYING for
+# Copyright 2001 Double Precision, Inc.  See COPYING for
 # distribution information.
 
 use webadmin;
@@ -11,7 +12,8 @@ use webadmin;
 my $errstr="";
 
 display_form("notsupp.html")
-    unless -f "$authmysqlrc";
+    unless -f "$libexecdir/authlib/authmysql" ||
+    -f "$libexecdir/authlib/authdaemond.mysql";
 
 if ($cgi->param("Save"))
 {
@@ -58,7 +60,7 @@ if ($cgi->param("Save"))
     {
 	$errstr="\@SAVED\@";
     }
-    changed("$authdaemond restart");
+    changed("test ! -x $libexecdir/authlib/authdaemond || $libexecdir/authlib/authdaemond restart");
 }
 
 my $mysql=ReadKWConfigFile("authmysqlrc");
