@@ -2,7 +2,7 @@
 #
 # TITLE: Inbound ESMTP
 #
-# $Id: admin-30esmtp.pl,v 1.11 2009/07/30 02:29:54 mrsam Exp $
+# $Id: admin-30esmtp.pl,v 1.10 2004/05/11 00:48:14 mrsam Exp $
 #
 # Copyright 2001-2004 Double Precision, Inc.  See COPYING for
 # distribution information.
@@ -137,10 +137,6 @@ if ($cgi->param("Save"))
     ReplaceEnvVarConfigFile("esmtpd-msa", "MAXPERC",
 			    $cgi->param("MAXPERC_MSA"));
 
-    $dodrop=$cgi->param("DROP") || "";
-
-    ReplaceEnvVarConfigFile("esmtpd", "DROP", $dodrop ? "-drop":"");
-
     my $esmtp=ReadEnvVarConfigFile("esmtpd");
 
 
@@ -351,7 +347,6 @@ if ($cgi->param("delacceptmailfor"))
 my $esmtp=ReadEnvVarConfigFile("esmtpd");
 my $ssl=ReadEnvVarConfigFile("esmtpd-ssl");
 my $msa=ReadEnvVarConfigFile("esmtpd-msa");
-my $dodrop=$$esmtp{"DROP"} || "";
 
 my $esmtpauth="";
 my $esmtpauth_tls="";
@@ -448,12 +443,6 @@ display_form("admin-30esmtp.html",
 	     {
 		 "ACCEPTMAILFORLIST" => $acceptHTML,
 		 "BLACKLISTS" => $blacklists,
-		 "NOTDROP" => "<input type=\"radio\" name='DROP' value=\"\""
-		     . (!$dodrop ? " checked=\"checked\"":"")
-		     . " />",
-		 "DROP" => "<input type=\"radio\" name='DROP' value=\"drop\""
-		     . ($dodrop ? " checked=\"checked\"":"")
-		     . " />",
 		 "ENABLED" => "<input type=\"checkbox\" name='ENABLED'" . (($$esmtp{"ESMTPDSTART"} =~ /[yY]/) ? " checked=\"checked\"":"") . " />",
 		 "ENABLED_SSL" => "<input type=\"checkbox\" name='ENABLED_SSL'" . ($$ssl{"ESMTPDSSLSTART"} =~ /[yY]/ ? " checked=\"checked\"":"") . " />",
 		 "ENABLED_MSA" => "<input type=\"checkbox\" name='ENABLED_MSA'" . ($$msa{"ESMTPDSTART"} =~ /[yY]/ ? " checked=\"checked\"":"") . " />",
