@@ -3,7 +3,7 @@
 ** distribution information.
 */
 
-static const char rcsid[]="$Id: mimegpgfork.c,v 1.3 2002/02/19 13:00:20 mrsam Exp $";
+static const char rcsid[]="$Id: mimegpgfork.c,v 1.5 2003/06/02 23:47:16 mrsam Exp $";
 
 #include "config.h"
 #include <stdio.h>
@@ -32,11 +32,10 @@ static const char rcsid[]="$Id: mimegpgfork.c,v 1.3 2002/02/19 13:00:20 mrsam Ex
 
 #include	"mimegpgfork.h"
 
-extern char *passphrase_fd;
-
-static int gpgmime_fork(const char *gpgdir,
-		 int xargc, char **xargv, int argc, char **argv,
-		 struct gpgmime_forkinfo *fi)
+static int libmail_gpgmime_fork(const char *gpgdir,
+			const char *passphrase_fd,
+			int xargc, char **xargv, int argc, char **argv,
+			struct gpgmime_forkinfo *fi)
 {
 	int pipes[3][2];
 	int n;
@@ -120,7 +119,7 @@ static int gpgmime_fork(const char *gpgdir,
 			}
 
 			newargv[i++]="--passphrase-fd";
-			newargv[i++]=passphrase_fd;
+			newargv[i++]=(char *)passphrase_fd;
 		}
 
 		for (n=0; n<xargc; n++)
@@ -178,7 +177,7 @@ static int gpgmime_fork(const char *gpgdir,
 
 static void gpgmime_writeflush(struct gpgmime_forkinfo *);
 
-void gpgmime_write(struct gpgmime_forkinfo *fi, const char *p, size_t n)
+void libmail_gpgmime_write(struct gpgmime_forkinfo *fi, const char *p, size_t n)
 {
 	while (n)
 	{
@@ -201,7 +200,7 @@ void gpgmime_write(struct gpgmime_forkinfo *fi, const char *p, size_t n)
 	}
 }
 
-static void gpgmime_read(struct gpgmime_forkinfo *fi, fd_set *fdr)
+static void libmail_gpgmime_read(struct gpgmime_forkinfo *fi, fd_set *fdr)
 {
 	char readbuf[BUFSIZ];
 	int i;
@@ -280,7 +279,7 @@ static void gpgmime_writeflush(struct gpgmime_forkinfo *fi)
 		if (select(maxfd+1, &fdr, &fdw, NULL, NULL) <= 0)
 			continue;
 
-		gpgmime_read(fi, &fdr);
+		libmail_gpgmime_read(fi, &fdr);
 
 		if (!FD_ISSET(fi->togpg_fd, &fdw))
 			continue;
@@ -298,7 +297,7 @@ static void gpgmime_writeflush(struct gpgmime_forkinfo *fi)
 	fi->gpg_writecnt=0;
 }
 
-int gpgmime_finish(struct gpgmime_forkinfo *fi)
+int libmail_gpgmime_finish(struct gpgmime_forkinfo *fi)
 {
 	pid_t p2;
 	int waitstat;
@@ -332,7 +331,7 @@ int gpgmime_finish(struct gpgmime_forkinfo *fi)
 		if (select(maxfd+1, &fdr, NULL, NULL, NULL) <= 0)
 			continue;
 
-		gpgmime_read(fi, &fdr);
+		libmail_gpgmime_read(fi, &fdr);
 	}
 
 	while ((p2=wait(&waitstat)) != fi->gpg_pid)
@@ -351,12 +350,14 @@ int gpgmime_finish(struct gpgmime_forkinfo *fi)
 	return (fi->gpg_errflag);
 }
 
-int gpgmime_fork_signencrypt(const char *gpgdir,
-			     int flags,
-			     int argc, char **argv,
-			     int (*output_func)(const char *, size_t, void *),
-			     void *output_voidarg,
-			     struct gpgmime_forkinfo *gpgfi)
+int libmail_gpgmime_forksignencrypt(const char *gpgdir,
+				    const char *passphrase_fd,
+				    int flags,
+				    int argc, char **argv,
+				    int (*output_func)(const char *,
+						       size_t, void *),
+				    void *output_voidarg,
+				    struct gpgmime_forkinfo *gpgfi)
 {
 	char *xargvec[5];
 	int xargc=0;
@@ -374,14 +375,16 @@ int gpgmime_fork_signencrypt(const char *gpgdir,
 
 	xargvec[xargc++]="-a";
 
-	rc=gpgmime_fork(gpgdir, xargc, xargvec, argc, argv, gpgfi);
+	rc=libmail_gpgmime_fork(gpgdir, passphrase_fd,
+			xargc, xargvec, argc, argv, gpgfi);
 
 	gpgfi->gpg_readhandler= output_func;
 	gpgfi->gpg_voidarg=output_voidarg;
 	return (rc);
 }
 
-int gpgmime_forkchecksign(const char *gpgdir,
+int libmail_gpgmime_forkchecksign(const char *gpgdir,
+			  const char *passphrase_fd,
 			  const char *content_filename,
 			  const char *signature_filename,
 			  int argc, char **argv,
@@ -408,12 +411,13 @@ int gpgmime_forkchecksign(const char *gpgdir,
 	newargv[i++]=(char *)signature_filename;
 	newargv[i++]=(char *)content_filename;
 
-	rc=gpgmime_fork(NULL, 3, xargvec, i, newargv, gpgfi);
+	rc=libmail_gpgmime_fork(gpgdir, passphrase_fd, 3, xargvec, i, newargv, gpgfi);
 	free(newargv);
 	return (rc);
 }
 
-int gpgmime_forkdecrypt(const char *gpgdir,
+int libmail_gpgmime_forkdecrypt(const char *gpgdir,
+			const char *passphrase_fd,
 			int argc, char **argv,
 			int (*output_func)(const char *, size_t, void *),
 			void *output_voidarg,
@@ -426,19 +430,19 @@ int gpgmime_forkdecrypt(const char *gpgdir,
 	xargv[1]="--charset";
 	xargv[2]=GPG_CHARSET;
 
-	rc=gpgmime_fork(NULL, 3, xargv, argc, argv, gpgfi);
+	rc=libmail_gpgmime_fork(gpgdir, passphrase_fd, 3, xargv, argc, argv, gpgfi);
 
 	gpgfi->gpg_readhandler= output_func;
 	gpgfi->gpg_voidarg= output_voidarg;
 	return (rc);
 }
 
-const char *gpgmime_getoutput(struct gpgmime_forkinfo *gpgfi)
+const char *libmail_gpgmime_getoutput(struct gpgmime_forkinfo *gpgfi)
 {
 	return (gpgfi->gpg_errbuf);
 }
 
-const char *gpgmime_getcharset(struct gpgmime_forkinfo *gpgfi)
+const char *libmail_gpgmime_getcharset(struct gpgmime_forkinfo *gpgfi)
 {
 	return (GPG_CHARSET);
 }
