@@ -1,11 +1,11 @@
 #ifndef tempname_h
 #define tempname_h
 /*
-** Copyright 2001 Double Precision, Inc.  See COPYING for
+** Copyright 2001-2003 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
-static const char tempname_h_rcsid[]="$Id: tempname.h,v 1.1 2001/03/15 04:20:35 mrsam Exp $";
+static const char tempname_h_rcsid[]="$Id: tempname.h,v 1.2 2003/06/02 15:51:23 mrsam Exp $";
 
 #include "config.h"
 
@@ -15,7 +15,7 @@ extern "C" {
 
 #define TEMPNAMEBUFSIZE	64
 
-int mimegpg_tempfile(char *);
+int libmail_tempfile(char *);
 
 #ifdef  __cplusplus
 } ;
