@@ -1,9 +1,9 @@
 /*
-** Copyright 2001 Double Precision, Inc.  See COPYING for
+** Copyright 2002 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
-static const char rcsid[]="$Id: testgpg.c,v 1.4 2001/10/01 22:51:46 mrsam Exp $";
+static const char rcsid[]="$Id: testgpg.c,v 1.6 2003/06/02 23:47:16 mrsam Exp $";
 
 
 #include	"config.h"
@@ -32,7 +32,7 @@ static int poll_wait(void *dummy)
 
 static void genkey(const char *d)
 {
-	gpg_genkey(d, &unicode_ISO8859_1,
+	libmail_gpg_genkey(d, &unicode_ISO8859_1,
 		   "John Smith",
 		   "john@example.com",
 		   "Dummy ISO-8859 Tst key",
@@ -64,27 +64,27 @@ static void genlist(const char *d, int flag)
 
 	memset(&gli, 0, sizeof(gli));
 
-	gpg_listkeys(d, flag, show_key, dump_stdout, &gli);
+	libmail_gpg_listkeys(d, flag, show_key, dump_stdout, &gli);
 }
 
 static int expkey(const char *d, const char *f, int flag)
 {
-	return (gpg_exportkey(d, flag, f, dump_stdout, dump_stdout, NULL));
+	return (libmail_gpg_exportkey(d, flag, f, dump_stdout, dump_stdout, NULL));
 }
 
 static int delkey(const char *d, const char *f, int flag)
 {
-	return (gpg_deletekey(d, flag, f, dump_stdout, NULL));
+	return (libmail_gpg_deletekey(d, flag, f, dump_stdout, NULL));
 }
 
 static int signkey(const char *d, const char *signthis, const char *signwith)
 {
-	return (gpg_signkey(d, signthis, signwith, -1, dump_stdout, NULL));
+	return (libmail_gpg_signkey(d, signthis, signwith, -1, dump_stdout, 0, NULL));
 }
 
 static int checksign(const char *d, const char *stuff, const char *sig)
 {
-	return (gpg_checksign(d, stuff, sig, dump_stdout, NULL));
+	return (libmail_gpg_checksign(d, stuff, sig, dump_stdout, NULL));
 }
 
 static int import(const char *filename)
@@ -98,18 +98,19 @@ static int import(const char *filename)
 		return (1);
 	}
 
-	if ((rc=gpg_import_start(NULL, 1)) == 0)
+	if ((rc=libmail_gpg_import_start(NULL, 1)) == 0)
 	{
 		char buf[BUFSIZ];
 		int n;
 
 		while ((n=fread(buf, 1, sizeof(buf), f)) > 0)
 		{
-			if ((rc=gpg_import_do(buf, n, dump_stdout, NULL)) != 0)
+			if ((rc=libmail_gpg_import_do(buf, n,
+						      dump_stdout, NULL)) != 0)
 				return (rc);
 		}
 
-		rc=gpg_import_finish(dump_stdout, NULL);
+		rc=libmail_gpg_import_finish(dump_stdout, NULL);
 	}
 	return (rc);
 }
