@@ -12,6 +12,7 @@
 #include	<errno.h>
 #include	<ctype.h>
 #include	<fcntl.h>
+#include	<time.h>
 #if	HAVE_UNISTD_H
 #include	<unistd.h>
 #endif
@@ -27,7 +28,7 @@
 #include	"tcpd/spipe.h"
 #include	"authlib/debug.h"
 
-static const char rcsid[]="$Id: imaplogin.c,v 1.23 2004/04/11 23:14:03 mrsam Exp $";
+static const char rcsid[]="$Id: imaplogin.c,v 1.24 2004/09/12 23:25:56 mrsam Exp $";
 
 FILE *debugfile=0;
 extern void mainloop();
@@ -39,6 +40,7 @@ unsigned long header_count=0, body_count=0;	/* Dummies */
 
 int main_argc;
 char **main_argv;
+extern time_t start_time;
 
 void rfc2045_error(const char *p)
 {
@@ -180,6 +182,7 @@ int do_imap_command(const char *tag)
 		putenv("IMAP_STARTTLS=NO");
 		putenv("IMAP_TLS_REQUIRED=0");
 		putenv("IMAP_TLS=1");
+
 		return (0);
 	}
 
@@ -272,6 +275,8 @@ const char	*ip;
 	ip=getenv("TCPREMOTEIP");
 	if (!ip)	exit(9);
 
+	time(&start_time);
+
 #if	IMAP_CLIENT_BUGS
 
 	ignorepunct();
