@@ -15,6 +15,7 @@
 #include	<string.h>
 #include	<sys/types.h>
 #include	<sys/time.h>
+#include	"numlib/numlib.h"
 #if	HAVE_UNISTD_H
 #include	<unistd.h>
 #endif
@@ -23,13 +24,15 @@
 #define	BUFSIZ	8192
 #endif
 
-static const char rcsid[]="$Id: imaptoken.c,v 1.17 2004/04/11 23:14:03 mrsam Exp $";
+static const char rcsid[]="$Id: imaptoken.c,v 1.18 2004/09/12 23:25:56 mrsam Exp $";
 
 static struct imaptoken curtoken;
 static char readbuf[BUFSIZ];
 
 char *imap_readptr=0;
 size_t imap_readptrleft=0;
+time_t start_time;
+
 static time_t readtimeout;
 
 extern FILE *debugfile;
@@ -38,17 +41,35 @@ extern unsigned long header_count, body_count;
 
 extern void bye();
 
-void disconnected()
+void bye_msg(const char *type)
 {
-const char *a=getenv("AUTHENTICATED");
+	const char *a=getenv("AUTHENTICATED");
+	char buf[NUMBUFSIZE];
+	const char *tls=getenv("IMAP_TLS");
+
+	libmail_str_time_t(time(NULL)-start_time, buf);
+
+	if (tls && atoi(tls))
+		tls=", starttls=1";
+	else
+		tls="";
 
 	if (a && *a)
-		fprintf(stderr, "ERR: DISCONNECTED, user=%s, "
-			"ip=[%s], headers=%lu, body=%lu\n",
-			a, getenv("TCPREMOTEIP"), header_count, body_count);
+		fprintf(stderr, "%s, user=%s, "
+			"ip=[%s], headers=%lu, body=%lu, time=%s%s\n",
+			type,
+			a, getenv("TCPREMOTEIP"), header_count, body_count,
+			buf, tls);
 	else
-		fprintf(stderr, "DEBUG: Disconnected, ip=[%s]\n",
-			getenv("TCPREMOTEIP"));
+		fprintf(stderr, "%s, ip=[%s], time=%s%s\n",
+			type,
+			getenv("TCPREMOTEIP"),
+			buf, tls);
+}
+
+void disconnected()
+{
+	bye_msg("ERR: DISCONNECTED");
 	bye();
 }
 
