@@ -1,8 +1,8 @@
 /*
-** Copyright 1998 - 2001 Double Precision, Inc.
+** Copyright 1998 - 2004 Double Precision, Inc.
 ** See COPYING for distribution information.
 **
-** $Id: pop3dserver.c,v 1.10 2001/12/29 04:01:39 mrsam Exp $
+** $Id: pop3dserver.c,v 1.31 2004/09/12 23:25:56 mrsam Exp $
 */
 
 #if	HAVE_CONFIG_H
@@ -48,14 +48,20 @@
 #endif
 #include	<signal.h>
 #include	<errno.h>
+#include	"authlib/auth.h"
 #include	"authlib/authmod.h"
 #include	"numlib/numlib.h"
 #include	"maildir/config.h"
 #include	"maildir/maildirmisc.h"
 #include	"maildir/maildirquota.h"
 #include	"maildir/maildirgetquota.h"
+#include	"maildir/maildircreate.h"
+#include	"maildir/loginexec.h"
+
+#define POP3DLIST "courierpop3dsizelist"
 
 extern void pop3dcapa();
+static void acctout(const char *disc);
 
 static const char *authaddr, *remoteip;
 
@@ -63,16 +69,49 @@ struct msglist {
 	struct msglist *next;
 	char *filename;
 	int isdeleted;
+	int isnew;
 	off_t size;
+
+	struct {
+		unsigned long uidv;
+		unsigned long n;
+	} uid;
 	} ;
 
 static struct msglist *msglist_l;
 static struct msglist **msglist_a;
 static unsigned msglist_cnt;
 
+static struct stat enomem_stat;
+static int enomem_1msg;
+
+/*
+** When a disk error occurs while saving an updated courierpop3dsizelist
+** file, proceed to recover as follows:
+**
+** If there's at least one existing message that's found in the old
+** courierpop3dsizelist, then ignore all new messages that were not found
+** in the old courierpop3dsizelist, and a new uid/uidv was assigned to them.
+** Therefore, the client doesn't see any new messages.  Hopefully, after
+** at least one existing message gets deleted, there'll be room to create
+** a new courierpop3dsizelist file next time, and record new messages.
+**
+** If none of the messages in the maildir could be found in the
+** courierpop3dsizelist, take the first message in the maildir only, and
+** use a UIDL that's derived from the message's dev_t/ino_t.  The client
+** will see that message only.  After deleting it, hopefully a new
+** courierpop3dsizelist file could be written out next time.
+*/
+
+
 static unsigned long top_count=0;
 static unsigned long retr_count=0;
 
+static unsigned long uidv=0;
+static int convert_v0=0;
+
+static time_t start_time;
+
 /*
 ** The RFC is pretty strict in stating that octet size must count the CR
 ** in the CRLF endofline.
@@ -86,7 +125,7 @@ int	c, lastc;
 	m->size=0;
 	if (f == 0)
 	{
-		perror("pop3d");
+		perror("calcsize fopen");
 		return;
 	}
 	lastc='\n';
@@ -100,15 +139,198 @@ int	c, lastc;
 
 	if (ferror(f))
 	{
-		perror("pop3d");
+		perror("calcsize ferror");
 		return;
 	}
 	fclose(f);
 }
 
+
+/*
+** Read courierpop3dsizelist
+*/
+
+static int cmpmsgs(const void *a, const void *b);
+
+static struct msglist **readpop3dlist(unsigned long *uid)
+{
+	struct msglist **a;
+	struct msglist *list=NULL;
+	size_t mcnt=0;
+
+	char linebuf[1024];
+
+	FILE *fp=fopen(POP3DLIST, "r");
+
+	size_t i;
+	int vernum=0;
+
+	uidv=time(NULL);
+
+	convert_v0=0;
+
+	if (fp == NULL ||
+	    fgets(linebuf, sizeof(linebuf)-1, fp) == NULL ||
+	    linebuf[0] != '/' || sscanf(linebuf+1, "%d %lu %lu", &vernum,
+					uid, &uidv)
+	    < 2 || (vernum != 1 && vernum != 2))
+	{
+		if (fp == NULL)
+			convert_v0=1;
+
+		if (vernum == 0 && fp && fseek(fp, 0L, SEEK_SET) >= 0)
+		{
+			/* Old version 0 format courierpop3dsizelist file */
+		}
+		else
+		{
+			if (fp)
+				fclose(fp);
+			fp=NULL;
+		}
+	}
+
+	if (fp)
+	{
+		struct msglist *m;
+
+		char *p, *q;
+
+		size_t n=0;
+		int ch;
+
+		while ((ch=getc(fp)) != EOF)
+		{
+			unsigned long sz;
+
+			if (ch != '\n')
+			{
+				if (n < sizeof(linebuf)-3)
+					linebuf[n++]=ch;
+				continue;
+			}
+			linebuf[n]=0;
+			n=0;
+
+			if (vernum == 0)
+				strcat(linebuf, " 0");
+			/* Convert version 0 to version 1 format - PRESTO! */
+
+			if ((p=strrchr(linebuf, ' ')) == NULL)
+				continue;
+			*p=0;
+			if ((q=strrchr(linebuf, ' ')) == NULL)
+				continue;
+			*p=' ';
+			p=q;
+			*p++=0;
+
+			if (linebuf[0] == 0)
+				continue;
+
+			if ((m=(struct msglist *)malloc(sizeof(struct
+							       msglist))) == 0)
+			{
+				perror("malloc");
+				exit(1);
+			}
+
+			if ((m->filename=strdup(linebuf)) == NULL)
+			{
+				perror("malloc");
+				exit(1);
+			}
+
+			switch (sscanf(p, "%lu %lu:%lu", &sz,
+				       &m->uid.n, &m->uid.uidv)) {
+			case 2:
+				m->uid.uidv=0;
+				/* FALLTHROUGH */
+			case 3:
+				m->size=sz;
+				m->next=list;
+				list=m;
+				++mcnt;
+				break;
+			default:
+				free(m->filename);
+				free(m);
+			}
+		}
+		fclose(fp);
+	}
+	if ((a=(struct msglist **)malloc((mcnt+1)
+					 *sizeof(struct msglist *))) == 0)
+	{
+		perror("malloc");
+		exit(1);
+	}
+
+	for (i=0; list; list=list->next)
+		a[i++]=list;
+
+	a[i]=NULL;
+	qsort(a, i, sizeof(*a), cmpmsgs);
+
+	return a;
+}
+
+static int savepop3dlist(struct msglist **a, size_t cnt,
+			  unsigned long uid)
+{
+	FILE *fp;
+	size_t i;
+
+	struct maildir_tmpcreate_info createInfo;
+
+	maildir_tmpcreate_init(&createInfo);
+
+	createInfo.uniq="pop3";
+	createInfo.doordie=1;
+
+	if ((fp=maildir_tmpcreate_fp(&createInfo)) == NULL)
+	{
+		maildir_tmpcreate_free(&createInfo);
+		return -1;
+	}
+
+	fprintf(fp, "/2 %lu %lu\n", uid, uidv);
+
+	for (i=0; i<cnt; i++)
+	{
+		char *p=a[i]->filename;
+		char *q;
+
+		if ((q=strrchr(p, '/')) != NULL)
+			p=q+1;
+
+		fprintf(fp, "%s %lu %lu:%lu\n", p, (unsigned long)a[i]->size,
+			a[i]->uid.n, a[i]->uid.uidv);
+	}
+
+	if (fflush(fp) || ferror(fp))
+	{
+		fclose(fp);
+		unlink(createInfo.tmpname);
+		maildir_tmpcreate_free(&createInfo);
+		return -1;
+	}
+
+	if (fclose(fp) ||
+	    rename(createInfo.tmpname, POP3DLIST) < 0)
+	{
+		unlink(createInfo.tmpname);
+		maildir_tmpcreate_free(&createInfo);
+		return -1;
+	}
+
+	maildir_tmpcreate_free(&createInfo);
+	return 0;
+}
+
 /* Scan cur, and pick up all messages contained therein */
 
-static void scancur()
+static int scancur()
 {
 DIR	*dirp;
 struct	dirent *de;
@@ -116,8 +338,8 @@ struct	msglist *m;
 
 	if ((dirp=opendir("cur")) == 0)
 	{
-		perror("pop3d");
-		return;
+		perror("scancur opendir(\"cur\")");
+		return 1;
 	}
 
 	while ((de=readdir(dirp)) != 0)
@@ -127,22 +349,22 @@ struct	msglist *m;
 		if ((m=(struct msglist *)malloc(sizeof(struct msglist))) == 0)
 		{
 			perror("malloc");
-			break;
+			exit(1);
 		}
 		if ((m->filename=(char *)malloc(strlen(de->d_name)+5)) == 0)
 		{
 			free( (char *)m);
 			perror("malloc");
-			break;
+			exit(1);
 		}
 		strcat(strcpy(m->filename, "cur/"), de->d_name);
 		m->isdeleted=0;
 		m->next=msglist_l;
 		msglist_l=m;
 		msglist_cnt++;
-		calcsize(m);
 	}
 	closedir(dirp);
+	return 0;
 }
 
 /*
@@ -152,43 +374,139 @@ struct	msglist *m;
 
 static int cmpmsgs(const void *a, const void *b)
 {
-const char *aname=(*(struct msglist **)a)->filename;
-const char *bname=(*(struct msglist **)b)->filename;
-const char *ap=strchr(aname, '/');
-const char *bp=strchr(bname, '/');
-int	n=0;
+	const char *aname=(*(struct msglist **)a)->filename;
+	const char *bname=(*(struct msglist **)b)->filename;
+	const char *ap=strrchr(aname, '/');
+	const char *bp=strrchr(bname, '/');
+	long	na, nb;
 
-	if (ap && bp)
+	if (ap)
+		++ap;
+	else
+		ap=aname;
+
+	if (bp)
+		++bp;
+	else
+		bp=bname;
+ 
+	na=atol(ap);
+	nb=atol(bp);
+
+	if (na < nb)	return (-1);
+	if (na > nb)	return (1);
+
+	while (*ap || *bp)
 	{
-	long	na=atol(ap+1);
-	long	nb=atol(bp+1);
+		if (*ap == ':' && *bp == ':')
+			break;
 
-		if (na < nb)	return (-1);
-		if (na > nb)	return (1);
-		n=strcmp(ap, bp);
+		if (*ap < *bp)
+			return -1;
+		if (*ap > *bp)
+			return 1;
+		++ap;
+		++bp;
 	}
-	if (n == 0)	n=strcmp(aname, bname);
-	return (n);
+
+	return 0;
 }
 
 static void sortmsgs()
 {
-unsigned i;
-struct msglist *m;
+	size_t i, n;
+	struct msglist *m;
+	struct msglist **prev_list;
+	int savesizes=0;
+
+	unsigned long nextuid;
 
 	if (msglist_cnt == 0)	return;
 
 	if ((msglist_a=(struct msglist **)malloc(
 			msglist_cnt*sizeof(struct msglist *))) == 0)
 	{
-		perror("pop3d");
+		perror("malloc");
 		msglist_cnt=0;
 		return;
 	}
 
 	for (i=0, m=msglist_l; m; m=m->next, i++)
+	{
+		m->isnew=0;
 		msglist_a[i]=m;
+	}
 	qsort(msglist_a, msglist_cnt, sizeof(*msglist_a), cmpmsgs);
+
+	nextuid=1;
+
+	prev_list=readpop3dlist(&nextuid);
+
+	n=0;
+
+	for (i=0; i<msglist_cnt; i++)
+	{
+		while (prev_list[n] &&
+		       cmpmsgs(&prev_list[n], &msglist_a[i]) < 0)
+		{
+			++n;
+			savesizes=1;
+		}
+
+		if (prev_list[n] &&
+		    cmpmsgs(&prev_list[n], &msglist_a[i]) == 0)
+		{
+			msglist_a[i]->size=prev_list[n]->size;
+			msglist_a[i]->uid=prev_list[n]->uid;
+			n++;
+		}
+		else
+		{
+			msglist_a[i]->uid.n=nextuid++;
+			msglist_a[i]->uid.uidv=uidv;
+			msglist_a[i]->isnew=1;
+			if (convert_v0)
+				msglist_a[i]->uid.n=0;
+
+			calcsize(msglist_a[i]);
+			savesizes=1;
+		}      
+	}
+
+	if (prev_list[n])
+		savesizes=1;
+
+	for (i=0; prev_list[i]; i++)
+	{
+		free(prev_list[i]->filename);
+		free(prev_list[i]);
+	}
+
+	free(prev_list);
+
+	if (savesizes && savepop3dlist(msglist_a, msglist_cnt, nextuid) < 0)
+	{
+		fprintf(stderr, "ERR: Error while saving courierpop3dsizelist"
+			", user=%s\n", authaddr);
+
+		for (i=n=0; i<msglist_cnt; i++)
+		{
+			if (msglist_a[i]->isnew)
+				continue;
+
+			msglist_a[n]=msglist_a[i];
+			++n;
+		}
+
+		if (n == 0 && n < msglist_cnt &&
+		    stat(msglist_a[0]->filename, &enomem_stat) == 0)
+		{
+			enomem_1msg=1;
+			++n;
+		}
+		msglist_cnt=n;
+
+	}
 }
 
 static void mkupper(char *p)
@@ -203,8 +521,6 @@ static void mkupper(char *p)
 static void cleanup()
 {
 	unsigned i;
-	char quotabuf[QUOTABUFSIZE];
-	int has_quota=maildir_getquota(".", quotabuf);
 
 	long deleted_bytes=0;
 	long deleted_messages=0;
@@ -216,7 +532,7 @@ static void cleanup()
 
 			const char *filename=msglist_a[i]->filename;
 
-			if (has_quota == 0 && !MAILDIR_DELETED(filename))
+			if (maildirquota_countfile(filename))
 			{
 				if (maildir_parsequota(filename, &un))
 				{
@@ -238,24 +554,8 @@ static void cleanup()
 		}
 
 	if (deleted_messages < 0)
-	{
-		int quotafd;
+		maildir_quota_deleted(".", deleted_bytes, deleted_messages);
 
-		if (maildir_checkquota(".", &quotafd, quotabuf, deleted_bytes,
-				       deleted_messages) && errno != EAGAIN &&
-		    deleted_bytes >= 0)
-		{
-			if (quotafd >= 0)
-				close (quotafd);
-		}
-		else
-		{
-			maildir_addquota(".", quotafd, quotabuf,
-					 deleted_bytes, deleted_messages);
-			if (quotafd >= 0)
-				close(quotafd);
-		}
-	}
 	return;
 }
 
@@ -275,7 +575,7 @@ char	buf[NUMBUFSIZE];
 		++j;
 	}
 
-	printf("+OK %u %s\r\n", j, str_off_t(n, buf));
+	printf("+OK %u %s\r\n", j, libmail_str_off_t(n, buf));
 	fflush(stdout);
 }
 
@@ -305,7 +605,7 @@ char	buf[NUMBUFSIZE];
 		if ((i=getmsgnum(msgnum)) != 0)
 		{
 			printf("+OK %u %s\r\n", i,
-				str_off_t(msglist_a[i-1]->size, buf));
+				libmail_str_off_t(msglist_a[i-1]->size, buf));
 			fflush(stdout);
 		}
 		return;
@@ -315,7 +615,7 @@ char	buf[NUMBUFSIZE];
 	for (i=0; i<msglist_cnt; i++)
 	{
 		if (msglist_a[i]->isdeleted)	continue;
-		printf("%u %s\r\n", i+1, str_off_t(msglist_a[i]->size, buf));
+		printf("%u %s\r\n", i+1, libmail_str_off_t(msglist_a[i]->size, buf));
 	}
 	printf(".\r\n");
 	fflush(stdout);
@@ -339,7 +639,7 @@ unsigned long *cntr;
 		return;
 	}
 	printf( (lptr ? "+OK headers follow.\r\n":"+OK %s octets follow.\r\n"),
-		str_off_t(msglist_a[i]->size, buf));
+		libmail_str_off_t(msglist_a[i]->size, buf));
 
 	cntr= &retr_count;
 	if (lptr)
@@ -365,13 +665,20 @@ unsigned long *cntr;
 		putchar(c);
 		++*cntr;
 	}
+	if (ferror(f)) {
+		/* Oops! All we can do is drop the connection */
+		fprintf(stderr, "ERR: I/O error while reading message file %s: %s\n",
+			msglist_a[i]->filename, strerror(errno));
+		acctout("INFO: I/O error disconnect");
+		exit(1);
+	}
 	if (lastc != '\n')	printf("\r\n");
 	printf(".\r\n");
 	fflush(stdout);
 	fclose(f);
 	if (lptr)	return;
 
-	if ((p=strchr(msglist_a[i]->filename, ':')) != 0 &&
+	if ((p=strchr(msglist_a[i]->filename, MDIRSEP[0])) != 0 &&
 		(p[1] != '2' || p[2] != ',' || strchr(p, 'S') != 0))
 		return;
 
@@ -379,7 +686,7 @@ unsigned long *cntr;
 		return;
 
 	strcpy(p, msglist_a[i]->filename);
-	if (strchr(p, ':') == 0)	strcat(p, ":2,");
+	if (strchr(p, MDIRSEP[0]) == 0)	strcat(p, MDIRSEP "2,");
 	strcat(p, "S");
 
 	if (lptr	/* Don't mark as seen for TOP */
@@ -393,14 +700,44 @@ unsigned long *cntr;
 }
 
 /*
-** The UIDL of the message is really just its filename, up to the first :
+** The UIDL of the message is really just its filename, up to the first MDIRSEP character
 */
 
 static void print_uidl(unsigned i)
 {
-const char *p=strchr(msglist_a[i]->filename, '/')+1;
+	const char *p;
 
-	while (*p && *p != ':')
+	if (enomem_1msg)
+		/* Error recovery - out of disk space, see comments
+		** at the beginning of this file.
+		*/
+	{
+		char dev_buf[NUMBUFSIZE];
+		char ino_buf[NUMBUFSIZE];
+		char mtime_buf[NUMBUFSIZE];
+
+		printf("ENOMEM-%s-%s-%s\r\n",
+		       libmail_strh_time_t(enomem_stat.st_mtime, mtime_buf),
+		       libmail_strh_dev_t(enomem_stat.st_dev, dev_buf),
+		       libmail_strh_ino_t(enomem_stat.st_ino, ino_buf));
+		return;
+	}
+
+	if (msglist_a[i]->uid.n != 0)
+	{
+		/* VERSION 1 and VERSION 2 UIDL */
+
+		printf((msglist_a[i]->uid.uidv ?
+			"UID%lu-%lu\r\n":"UID%lu\r\n"),
+		       msglist_a[i]->uid.n, msglist_a[i]->uid.uidv);
+		return;
+	}
+
+	/* VERSION 0 UIDL */
+
+	p=strchr(msglist_a[i]->filename, '/')+1;
+
+	while (*p && *p != MDIRSEP[0])
 	{
 		if (*p < 0x21 || *p > 0x7E || *p == '\'' || *p == '"' ||
 			*p == '+')
@@ -443,12 +780,19 @@ static void acctout(const char *disc)
 	static const char msg3[]=", ip=[";
 	static const char msg4[]="], top=";
 	static const char msg5[]=", retr=";
+	static const char msg6[]=", time=";
+	static const char msg7[]=", stls=1";
+
 	char num1[NUMBUFSIZE];
 	char num2[NUMBUFSIZE];
-	char *p;
+	char num3[NUMBUFSIZE];
 
-	str_size_t(top_count, num1);
-	str_size_t(retr_count, num2);
+	char *p;
+	const char *q;
+
+	libmail_str_size_t(top_count, num1);
+	libmail_str_size_t(retr_count, num2);
+	libmail_str_time_t(time(NULL)-start_time, num3);
 
 	p=malloc(strlen(authaddr)+strlen(remoteip)+strlen(disc)+
 		 strlen(num1)+strlen(num2)+200);	/* Should be enough */
@@ -462,6 +806,12 @@ static void acctout(const char *disc)
 	strcat(p, num1);
 	strcat(p, msg5);
 	strcat(p, num2);
+	strcat(p, msg6);
+	strcat(p, num3);
+
+	if ((q=getenv("POP3_STLS")) && atoi(q))
+		strcat(p, msg7);
+
 	strcat(p, "\n");
 	write(2, p, strlen(p));
 	free(p);
@@ -620,29 +970,6 @@ char	*n;
 	closedir(p);
 }
 
-static void scannew()
-{
-DIR	*p=opendir("new");
-struct	dirent *de;
-char	*n, *o;
-
-	if (!p)	return;
-
-	while ((de=readdir(p)) != 0)
-	{
-		if (de->d_name[0] == '.')	continue;
-		n=malloc(strlen(de->d_name)+5);
-		if (!n)	continue;
-		o=malloc(strlen(de->d_name)+8);
-		if (!o)	continue;
-		strcat(strcpy(n, "new/"), de->d_name);
-		strcat(strcat(strcpy(o, "cur/"), de->d_name), ":2,");
-		rename(n, o);
-		free(n);
-		free(o);
-	}
-	closedir(p);
-}
 
 #include	<unistd.h>
 
@@ -651,6 +978,29 @@ int main(int argc, char **argv)
 char	*p;
 
 	authmodclient();
+
+	time(&start_time);
+
+	if ((authaddr=getenv("AUTHADDR")) == 0 ||
+		(remoteip=getenv("TCPREMOTEIP")) == 0)
+	{
+		fprintf(stderr, "ERROR: Required environment variables not initialized.\n");
+		fflush(stderr);
+		exit (1);
+	}
+
+	{
+	struct	stat	buf;
+
+		if ( stat(".", &buf) < 0 || buf.st_mode & S_ISVTX)
+		{
+			fprintf(stderr, "INFO: LOCKED, user=%s, ip=[%s]\n",
+				authaddr, remoteip);
+			printf("-ERR Your account is temporarily unavailable (+t bit set on home directory).\r\n");
+			exit(0);
+		}
+	}
+
 	if ((p=getenv("MAILDIR")) != 0 && *p)
 	{
 		if (chdir(p))
@@ -664,14 +1014,21 @@ char	*p;
 		printf("-ERR Maildir: %s\r\n",strerror(errno));
 		exit(1);
 	}
-	if ((authaddr=getenv("AUTHADDR")) == 0 ||
-		(remoteip=getenv("TCPREMOTEIP")) == 0)
+	
+	maildir_loginexec();
+
+	p=authgetoptionenv("disablepop3");
+
+	if (p && atoi(p))
 	{
-		fprintf(stderr, "ERROR: Required environment variables not initialized.\n");
-		fflush(stderr);
-		exit (1);
+		printf("-ERR POP3 access disabled for this account.\r\n");
+		fflush(stdout);
+		exit(1);
 	}
 
+	free(p);
+
+
 	fprintf(stderr, "INFO: LOGIN, user=%s, ip=[%s]\n",
 			authaddr,
 			remoteip);
@@ -681,8 +1038,12 @@ char	*p;
 	msglist_l=0;
 	msglist_a=0;
 	purgetmp();
-	scannew();
-	scancur();
+	maildir_getnew(".", INBOX, NULL, NULL);
+	if (scancur())
+	{
+		printf("-ERR Maildir invalid (no 'cur' directory)\r\n");
+		return (0);
+	}
 	sortmsgs();
 	printf("+OK logged in.\r\n");
 	fflush(stdout);
