@@ -26,8 +26,7 @@ FILE	*fp;
 	{
 		if (strcmp(b->name, delname) == 0)	continue;
 
-		fprintf(fp, "%s\t%s\t%s\t%s\t%s\t%s\n", b->name, b->host,
-			b->port, b->suffix, b->binddn, b->bindpw);
+		ldapabook_writerec(b, fp);
 	}
 	ldapabook_free(a);
 
