@@ -48,7 +48,7 @@
 #include	<ctype.h>
 #include	<numlib/numlib.h>
 
-static const char rcsid[]="$Id: maildirquota.c,v 1.24 2004/05/25 02:08:40 mrsam Exp $";
+static const char rcsid[]="$Id: maildirquota.c,v 1.25 2004/06/18 23:43:40 mrsam Exp $";
 
 static void parsequotastr(const char *, struct maildirquota *);
 
@@ -748,7 +748,7 @@ struct	stat	stat_buf;
 char	*p;
 DIR	*dirp;
 struct dirent *de;
-off_t	s;
+unsigned long s;
 
 	if (stat(dir, &stat_buf))	return (0);	/* Ignore */
 	if (stat_buf.st_mtime > *dirstamp)	*dirstamp=stat_buf.st_mtime;
