@@ -1,9 +1,9 @@
 /*
-** Copyright 2001 Double Precision, Inc.  See COPYING for
+** Copyright 2001-2003 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
-static const char rcsid[]="$Id: pcpd.c,v 1.7 2002/02/13 04:03:29 mrsam Exp $";
+static const char rcsid[]="$Id: pcpd.c,v 1.16 2004/05/02 14:59:42 mrsam Exp $";
 
 #include "config.h"
 #include "pcp.h"
@@ -20,9 +20,6 @@ static const char rcsid[]="$Id: pcpd.c,v 1.7 2002/02/13 04:03:29 mrsam Exp $";
 #if HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
-#if HAVE_SYSLOG_H
-#include <syslog.h>
-#endif
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/socket.h>
@@ -30,7 +27,7 @@ static const char rcsid[]="$Id: pcpd.c,v 1.7 2002/02/13 04:03:29 mrsam Exp $";
 #include <sys/un.h>
 #include "liblock/config.h"
 #include "liblock/liblock.h"
-#include "authlib/auth.h"
+#include "numlib/numlib.h"
 #include "maildir/maildircache.h"
 #include "pcpdauth.h"
 #include "pcpdauthtoken.h"
@@ -38,6 +35,8 @@ static const char rcsid[]="$Id: pcpd.c,v 1.7 2002/02/13 04:03:29 mrsam Exp $";
 
 PCP_STRERROR
 
+#define exit(_a_) _exit(_a_)
+
 static char *userid;
 static char *proxy_userid;
 static char *deleted_eventid;
@@ -295,7 +294,7 @@ static int addrcmp(const char *a, const char *b)
 
 		if (!aa)
 		{
-			syslog(LOG_NOTICE, "malloc: out of memory.");
+			fprintf(stderr, "NOTICE: malloc: out of memory.\n");
 			return (1);
 		}
 		strcat(strcat(strcpy(aa, a), "@"), h);
@@ -310,7 +309,7 @@ static int addrcmp(const char *a, const char *b)
 
 		if (!aa)
 		{
-			syslog(LOG_NOTICE, "malloc: out of memory.");
+			fprintf(stderr, "NOTICE: malloc: out of memory.\n");
 			return (1);
 		}
 		strcat(strcat(strcpy(aa, b), "@"), h);
@@ -577,7 +576,7 @@ static struct PCP_new_eventid *readnewevent(struct PCP *pcp)
 
 		if (!n)
 		{
-			syslog(LOG_ALERT, "Out of memory.");
+			fprintf(stderr, "ALERT: Out of memory.\n");
 			exit(1);
 		}
 
@@ -1714,8 +1713,8 @@ static int doline(struct PCP *pcp, char *p, int acl_flags)
 						       pl->newevent,
 						       &new_commit))
 					{
-						syslog(LOG_CRIT,
-						       "COMMIT failed for PROXY %s",
+						fprintf(stderr, "CRIT: "
+						       "COMMIT failed for PROXY %s\n",
 						       pl->userid);
 
 						if (!force_flag)
@@ -1738,8 +1737,8 @@ static int doline(struct PCP *pcp, char *p, int acl_flags)
 
 					if (pcp_delete(pl->proxy, &del))
 					{
-						syslog(LOG_CRIT,
-						       "DELETE failed for PROXY %s",
+						fprintf(stderr, "CRIT: "
+						       "DELETE failed for PROXY %s\n",
 						       pl->userid);
 						if (!force_flag)
 						{
@@ -1803,8 +1802,8 @@ static int doline(struct PCP *pcp, char *p, int acl_flags)
 
 					if (pcp_delete(pl->proxy, &del))
 					{
-						syslog(LOG_CRIT,
-						       "DELETE failed for PROXY %s",
+						fprintf(stderr, "CRIT: "
+						       "DELETE failed for PROXY %s\n",
 						       pl->userid);
 					}
 				}
@@ -2058,16 +2057,9 @@ static void accept_pcpd(int, int, int, int);
 
 static void start()
 {
-	int lockfd=ll_daemon_start(LOCKFILE);
 	int pubsock;
 	int privsock;
 
-	if (lockfd < 0)
-	{
-		perror(LOCKFILE);
-		exit(1);
-	}
-
 	if ((pubsock=pcp_mksocket(PUBDIR, "PCPDLOCAL")) < 0)
 	{
 		exit(1);
@@ -2078,9 +2070,18 @@ static void start()
 		exit (1);
 	}
 
-	ll_daemon_resetio();
-	ll_daemon_started(PIDFILE, lockfd);
+#if USE_NOCLDWAIT
+	{
+		struct sigaction sa;
+
+		memset(&sa, 0, sizeof(sa));
+		sa.sa_handler=SIG_IGN;
+		sa.sa_flags=SA_NOCLDWAIT;
+		sigaction(SIGCHLD, &sa, NULL);
+	}
+#else
 	signal(SIGCHLD, SIG_IGN);
+#endif
 
 	for (;;)
 	{
@@ -2167,7 +2168,7 @@ static int callback_login(struct userid_callback *a, void *vp)
 		token=authtoken_create(uinfo->userid, t);
 		if (!token)
 		{
-			syslog(LOG_NOTICE, "pcpd: authtoken_create() failed.");
+			fprintf(stderr, "NOTICE: authtoken_create() failed.\n");
 			maildir_cache_cancel();
 			return (1);
 		}
@@ -2175,12 +2176,13 @@ static int callback_login(struct userid_callback *a, void *vp)
 
 	maildir_cache_start();
 
-	authchangeuidgid(a->uid, getgid());
+	libmail_changeuidgid(a->uid, getgid());
 
 	if (chdir(a->homedir) < 0)
 	{
 		free(token);
-		syslog(LOG_NOTICE, "pcpd: chdir(%s) failed.", a->homedir);
+		fprintf(stderr, "NOTICE: chdir(%s) failed: %s\n", a->homedir,
+			strerror(errno));
 		maildir_cache_cancel();
 		exit(1);
 	}
@@ -2188,7 +2190,8 @@ static int callback_login(struct userid_callback *a, void *vp)
 	if (chdir(a->maildir && *a->maildir ? a->maildir:"Maildir") < 0)
 	{
 		free(token);
-		syslog(LOG_NOTICE, "pcpd: chdir(Maildir) failed.");
+		fprintf(stderr, "NOTICE: chdir(Maildir) failed: %s\n",
+			strerror(errno));
 		maildir_cache_cancel();
 		exit(1);
 	}
@@ -2197,7 +2200,8 @@ static int callback_login(struct userid_callback *a, void *vp)
 	if (chdir("calendar") < 0)
 	{
 		free(token);
-		syslog(LOG_NOTICE, "pcpd: chdir(Maildir) failed.");
+		fprintf(stderr, "NOTICE: chdir(calendar) failed: %s\n",
+			strerror(errno));
 		maildir_cache_cancel();
 		exit(1);
 	}
@@ -2205,7 +2209,8 @@ static int callback_login(struct userid_callback *a, void *vp)
 	curdir[sizeof(curdir)-1]=0;
 	if (getcwd(curdir, sizeof(curdir)-1) == 0)
 	{
-		syslog(LOG_NOTICE, "pcpd: getcwd() failed.");
+		fprintf(stderr, "NOTICE: getcwd() failed: %s\n",
+			strerror(errno));
 		maildir_cache_cancel();
 		exit(1);
 	}
@@ -2244,7 +2249,7 @@ static void accept_pcpd(int sock, int pubsock, int privsock, int flag)
 
 	if (fcntl(fd, F_SETFL, 0) < 0)
 	{
-		syslog(LOG_CRIT, "pcpd: fcntl() failed: %m");
+		fprintf(stderr, "CRIT: fcntl() failed: %s\n", strerror(errno));
 		close(fd);
 		return;
 	}
@@ -2254,7 +2259,7 @@ static void accept_pcpd(int sock, int pubsock, int privsock, int flag)
 
 	if (pid < 0)
 	{
-		syslog(LOG_CRIT, "pcpd: fork() failed: %m");
+		fprintf(stderr, "CRIT: fork() failed: %s\n", strerror(errno));
 		close(fd);
 		return;
 	}
@@ -2266,6 +2271,7 @@ static void accept_pcpd(int sock, int pubsock, int privsock, int flag)
 	}
 
 	/* child */
+	signal(SIGCHLD, SIG_DFL);
 
 	close(pubsock);
 	close(privsock);
@@ -2284,13 +2290,13 @@ static void accept_pcpd(int sock, int pubsock, int privsock, int flag)
 	if (pcp && flag && pcp_cleanup(pcp))
 	{
 		pcp_close(pcp);
-		syslog(LOG_CRIT, "pcpd: pcp_cleanup failed");
+		fprintf(stderr, "CRIT: pcp_cleanup failed\n");
 		pcp=NULL;
 	}
 
 	if (!pcp)
 	{
-		syslog(LOG_CRIT, "pcpd: pcp_open_dir failed");
+		fprintf(stderr, "CRIT: pcp_open_dir failed\n");
 		perror("pcp_open_dir");
 		exit(1);
 	}
@@ -2324,20 +2330,20 @@ static int callback_cache_search(uid_t u, gid_t g, const char *dir, void *vp)
 			token=authtoken_create(rs->userid, now);
 			if (!token)
 			{
-				syslog(LOG_ALERT,
-				       "pcpd: authtoken_create() failed.");
+				fprintf(stderr,
+				       "ALERT: authtoken_create() failed.\n");
 				return (1);
 			}
 		}
 		maildir_cache_start();
 	}
 
-	authchangeuidgid(u, g);
+	libmail_changeuidgid(u, g);
 
 	if (chdir(dir) < 0)
 	{
 		maildir_cache_cancel();
-		syslog(LOG_NOTICE, "pcpd: chdir(%s) failed.", dir);
+		fprintf(stderr, "NOTICE: chdir(%s) failed: %s\n", dir, strerror(errno));
 		return (-1);
 	}
 
@@ -2501,8 +2507,8 @@ static char *login(int isprivate,
 					       &uinfo);
 				if (rc)
 				{
-					syslog(LOG_CRIT,
-					       "pcpd: auth_userid() failed\n");
+					fprintf(stderr,
+					       "CRIT: auth_userid() failed\n");
 					exit(1);
 				}
 				alarm(0);
@@ -2547,8 +2553,8 @@ static char *login(int isprivate,
 
 				if (rc)
 				{
-					syslog(LOG_NOTICE,
-					       "pcpd: auth_userid() failed.");
+					fprintf(stderr,
+					       "NOTICE: auth_userid() failed.\n");
 					printf("400 Internal failure - try again later.\n");
 					continue;
 				}
@@ -2585,7 +2591,7 @@ int main(int argc, char **argv)
 
 	maildir_cache_init(TIMEOUT * 2, CACHEDIR, LOCALCACHEOWNER, authvars);
 
-	if (strcmp(argv[argn], "start") == 0)
+	if (strcmp(argv[argn], "server") == 0)
 	{
 		struct group *gr;
 
@@ -2603,7 +2609,7 @@ int main(int argc, char **argv)
 		}
 
 		authtoken_init();
-		authchangeuidgid(getuid(), gr->gr_gid);
+		libmail_changeuidgid(getuid(), gr->gr_gid);
 		start();
 	}
 	else if (strcmp(argv[argn], "login") == 0 ||
@@ -2620,7 +2626,7 @@ int main(int argc, char **argv)
 			fprintf(stderr, "Unknown group: %s\n", MAILGROUP);
 			exit(1);
 		}
-		authchangeuidgid(getuid(), gr->gr_gid);
+		libmail_changeuidgid(getuid(), gr->gr_gid);
 
 		if (chdir(CALENDARDIR) < 0)
 		{
@@ -2636,13 +2642,13 @@ int main(int argc, char **argv)
 		if (pcp && flag && pcp_cleanup(pcp))
 		{
 			pcp_close(pcp);
-			syslog(LOG_CRIT, "pcpd: pcp_cleanup failed");
+			fprintf(stderr, "CRIT: pcp_cleanup failed\n");
 			pcp=NULL;
 		}
 
 		if (!pcp)
 		{
-			syslog(LOG_CRIT, "pcpd: pcp_open_dir failed");
+			fprintf(stderr, "CRIT: pcp_open_dir failed\n");
 			perror("pcp_open_dir");
 			exit(1);
 		}
@@ -2650,16 +2656,6 @@ int main(int argc, char **argv)
 		mainloop(pcp);
 		exit(0);
 	}
-	else if (strcmp(argv[argn], "stop") == 0)
-	{
-		if (chdir(CALENDARDIR) < 0)
-		{
-			perror(CALENDARDIR);
-			exit(1);
-		}
-		ll_daemon_stop(LOCKFILE, PIDFILE);
-		exit(0);
-	}
 	else if (strcmp(argv[argn], "open") == 0)
 	{
 		++argn;
@@ -2673,7 +2669,7 @@ int main(int argc, char **argv)
 			exit(0);
 		}
 	}
-	fprintf(stderr, "Usage: %s (start|stop|open [path])\n", argv[0]);
-	exit(1);
+	fprintf(stderr, "Usage: %s (server|open [path])\n", argv[0]);
+	exit(0); /* exit(1) breaks Courier rpm %preun script */
 	return (0);
 }
