@@ -16,9 +16,6 @@
 */
 
 #include	"soxwrap/soxwrap.h"
-#ifdef	getc
-#undef	getc
-#endif
 #include	<stdio.h>
 
 
@@ -55,7 +52,7 @@
 
 #include	<netdb.h>
 
-static const char rcsid[]="$Id: tcpd.c,v 1.37 2004/01/24 20:10:57 mrsam Exp $";
+static const char rcsid[]="$Id: tcpd.c,v 1.38 2004/07/24 03:36:53 mrsam Exp $";
 
 static const char *accessarg=0;
 static const char *denymsgarg=0;
