@@ -1,6 +1,6 @@
 #! perl
 #
-# $Id: admin-main.pl,v 1.2 2003/07/06 16:16:37 mrsam Exp $
+# $Id: admin-main.pl,v 1.3 2004/08/26 02:16:33 mrsam Exp $
 #
 # Copyright 2001 Double Precision, Inc.  See COPYING for
 # distribution information.
@@ -45,8 +45,18 @@ my $menuhtml="<table border=\"0\" cellspacing=\"8\">";
 
 foreach (sort keys %mainmenu)
 {
-    $menuhtml .= "<tr><td><a href=\""
-	. $cgi->url(-full=>1) . "/$_\">" . $mainmenu{$_} . "</a></td></tr>\n";
+    my $title=$mainmenu{$_};
+
+    my $pfix;
+
+    $title =~ /^(>*)/;
+
+    $pfix=length($1);
+
+    $title =~ s/^>*//;
+
+    $menuhtml .= "<tr><td>" . ("&nbsp;" x $pfix) . "<a href=\""
+	. $cgi->url(-full=>1) . "/$_\">" . $title . "</a></td></tr>\n";
 }
 
 $menuhtml .= "</table>\n";
