@@ -4,7 +4,7 @@
 */
 
 /*
-** $Id: addressbook.c,v 1.10 2001/12/07 12:54:59 mrsam Exp $
+** $Id: addressbook.c,v 1.12 2003/06/17 15:38:37 mrsam Exp $
 */
 
 #include	"sqwebmail.h"
@@ -410,7 +410,7 @@ size_t	acnt=0, i;
 			output_attrencoded_fp(aa[i]->name, w);
 			fprintf(w, "\">");
 			output_attrencoded_fp(aa[i]->name, w);
-			fprintf(w, "\n");
+			fprintf(w, "</option>\n");
 		}
 		free(aa);
 		abl_free(a);
@@ -533,7 +533,7 @@ void ab_addrselect()
 {
 	struct ab_addrselect_s *list=NULL, *p;
 
-	printf("<select width=20 name=addressbookname><option value=""></option>\n");
+	printf("<select name=\"addressbookname\"><option value=\"\"></option>\n");
 
 	if (ab_get_nameaddr(ab_addrselect_cb, &list) == 0)
 	{
@@ -632,7 +632,7 @@ int	replace_index=0;
 			for (i=0; i<counter; i++)
 			{
 			const char *addy=cgi(strcat(strcpy(numbuf2, "ADDY"),
-                                        str_size_t(i, numbuf)));
+                                        libmail_str_size_t(i, numbuf)));
 			char	*addycpy;
 			char	*name;
 
@@ -706,7 +706,7 @@ int	replace_index=0;
 	printf("%s", nick_prompt);
 
 	printf("<select name=\"nick\">\n");
-	printf("<option value=\"\">\n");
+	printf("<option value=\"\"></option>\n");
 
 	ab_listselect();
 
@@ -727,11 +727,11 @@ int	replace_index=0;
 	if (do_edit && *s)
 	{
 		printf("%s%s%s", nick_title1, s, nick_title2);
-		printf("<input type=hidden name=\"editnick\" value=\"");
+		printf("<input type=\"hidden\" name=\"editnick\" value=\"");
 		output_attrencoded(s);
-		printf("\">\n");
+		printf("\" />\n");
 
-		printf("<table border=0 align=center>");
+		printf("<table border=\"0\" align=\"center\">");
 
 
 		if ((fp=fopen(ADDRESSBOOK, "r")) != 0)
@@ -803,7 +803,7 @@ int	replace_index=0;
 
 						if (a->addrs[i].tokens == 0)
 							continue;
-						printf("<tr><td align=right"
+						printf("<tr><td align=\"right\""
 						       " class=\"nickname\">");
 
 						s=rfc822_getname(a, i);
@@ -818,7 +818,7 @@ int	replace_index=0;
 							free(s);
 							printf("\"");
 						}
-						printf("</td><td align=left"
+						printf("</td><td align=\"left\""
 						       " class=\"nickaddr\">"
 						       "&lt;");
 						s=rfc822_getaddr(a, i);
@@ -827,7 +827,7 @@ int	replace_index=0;
 							output_attrencoded(s);
 							free(s);
 						}
-						printf("&gt;</td><td><input type=submit name=\"startedit%d\" value=\"%s\">&nbsp;<input type=submit name=\"del%d\" value=\"%s\"></td></tr>\n",
+						printf("&gt;</td><td><input type=\"submit\" name=\"startedit%d\" value=\"%s\" />&nbsp;<input type=\"submit\" name=\"del%d\" value=\"%s\" /></td></tr>\n",
 							i, nick_edit,
 							i, nick_delete);
 					}
@@ -838,8 +838,8 @@ int	replace_index=0;
 			}
 			fclose(fp);
 		}
-		printf("<tr><td colspan=3><hr width=\"90%%\"></td></tr>\n");
-		printf("<tr><td align=right>%s</td><td colspan=2><input type=text name=newname", nick_name);
+		printf("<tr><td colspan=\"3\"><hr width=\"90%%\" /></td></tr>\n");
+		printf("<tr><td align=\"right\">%s</td><td colspan=\"2\"><input type=\"text\" name=\"newname\"", nick_name);
 
 		if (edit_name)
 		{
@@ -847,9 +847,9 @@ int	replace_index=0;
 			output_attrencoded(edit_name);
 			printf("\"");
 		}
-		printf("></td></tr>\n");
+		printf(" /></td></tr>\n");
 
-		printf("<tr><td align=right>%s</td><td><input type=text name=newaddress", nick_address);
+		printf("<tr><td align=\"right\">%s</td><td><input type=\"text\" name=\"newaddress\"", nick_address);
 		if (edit_addr)
 		{
 			printf(" value=\"");
@@ -857,13 +857,13 @@ int	replace_index=0;
 			printf("\"");
 		}
 
-		printf("></td><td>");
+		printf(" /></td><td>");
 
 		if (edit_name || edit_addr)
-			printf("<input type=hidden name=replacenick value=%d>",
+			printf("<input type=\"hidden\" name=\"replacenick\" value=\"%d\" />",
 				replace_index);
 
-		printf("<input type=submit name=add value=\"%s\"></td></tr>\n",
+		printf("<input type=\"submit\" name=\"add\" value=\"%s\" /></td></tr>\n",
 			edit_name || edit_addr ? nick_edit:nick_add);
 
 		printf("</table>");
