@@ -1,11 +1,11 @@
 /*
-** Copyright 1998 - 2001 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2002 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
 
 /*
-** $Id: attachments.c,v 1.31 2002/01/13 18:22:50 mrsam Exp $
+** $Id: attachments.c,v 1.41 2004/05/23 14:28:24 mrsam Exp $
 */
 #include	"config.h"
 #include	"sqwebmail.h"
@@ -41,15 +41,13 @@
 #ifndef WIFEXITED
 #define WIFEXITED(stat_val) (((stat_val) & 255) == 0)
 #endif
-#if	HAVE_SYSLOG_H
-#include	<syslog.h>
-#else
-#define syslog(a,b)
-#endif
 
 #include	"maildir/maildirmisc.h"
 
 #include	"htmllibdir.h"
+#if	HAVE_SQWEBMAIL_UNICODE
+#include	"unicode/unicode.h"
+#endif
 
 extern char *newmsg_alladdrs(FILE *);
 
@@ -70,6 +68,7 @@ extern void newmsg_create_multipart(int, const char *, const char *);
 extern void newmsg_copy_nonmime_headers(FILE *);
 
 extern const char *sqwebmail_content_charset;
+extern const char *sqwebmail_content_language;
 
 static void attachment_showname(const char *);
 
@@ -88,9 +87,12 @@ const char	*quotaerr=getarg("QUOTAERR");
 const char	*limiterr=getarg("LIMITERR");
 off_t	dummy;
 int	fd2;
+#if	HAVE_SQWEBMAIL_UNICODE
+const struct unicode_info *uiptr=unicode_find(sqwebmail_content_charset);
+#endif
 
 	CHECKFILENAME(draft);
-	filename=maildir_find(DRAFTS, draft);
+	filename=maildir_find(INBOX "." DRAFTS, draft);
 	if (!filename)	return;
 
 	fd2=maildir_safeopen(filename, O_RDONLY, 0);
@@ -130,7 +132,7 @@ int	fd2;
 	newmsg_hiddenheader("pos", pos);
 	newmsg_hiddenheader("draft", draft);
 	tokennew();
-	printf("<table width=\"100%%\" border=0>");
+	printf("<table width=\"100%%\" border=\"0\">");
 
 	for (q=rfcp ? rfcp->firstpart:0; q; q=q->next)
 	{
@@ -138,6 +140,9 @@ int	fd2;
 	const char *content_transfer_encoding;
 	const char *charset;
 	const char *name;
+	const char *cn;
+	char *content_name;
+
 	off_t start_pos, end_pos, start_body;
 
 		if (q->isdummy)	continue;
@@ -153,22 +158,54 @@ int	fd2;
 		rfc2045_mimepos(q, &start_pos, &end_pos, &start_body,
 			&dummy, &dummy);
 
-		printf("<TR><TD ALIGN=LEFT><INPUT TYPE=CHECKBOX NAME=\"del%d\">&nbsp;",
-			++cnt);
-		name=rfc2045_contentname(q);
-		if ((!name || !*name) &&
+		++cnt;
+		printf("<tr><td align=\"left\"><input type=\"checkbox\" name=\"del%d\" id=\"del%d\" />&nbsp;",
+			cnt, cnt);
+
+#if	HAVE_SQWEBMAIL_UNICODE
+		if (rfc2231_udecodeType(q, "name", uiptr, &content_name) < 0 ||
+		    rfc2231_udecodeDisposition(q, "filename", uiptr,
+					       &content_name) < 0)
+			content_name=NULL;
+#else
+		if (rfc2231_decodeType(q, "name", NULL, NULL, &content_name) < 0 ||
+		    rfc2231_decodeDisposition(q, "filename", NULL, NULL,
+					     &content_name) < 0)
+			content_name=NULL;
+#endif
+		if (!content_name &&
+		    ((cn=rfc2045_getattr(q->content_type_attr, "name")) ||
+		     (cn=rfc2045_getattr(q->content_disposition_attr,
+					 "filename"))) &&
+		    strstr(cn, "=?") && strstr(cn, "?="))
+		    /* RFC2047 header encoding (not compliant to RFC2047) */
+#if	HAVE_SQWEBMAIL_UNICODE
+			content_name = rfc2047_decode_unicode(cn, uiptr,
+				RFC2047_DECODE_ABORT);
+#else
+			content_name = rfc2047_decode_enhanced(cn,
+				sqwebmail_content_charset);
+#endif
+
+		if ((!content_name || !*content_name) &&
 		    strcmp(content_type, "application/pgp-keys") == 0)
 			name=getarg("KEYDESCR");
+		else
+		{
+			name=content_name;
+		}
 
 		attachment_showname(name);
-		printf("</TD><TD ALIGN=LEFT>&nbsp;&nbsp;");
+		if (content_name)
+			free(content_name);
+		printf("</td><td align=\"left\">&nbsp;&nbsp;<label for=\"del%d\">", cnt);
 		output_attrencoded( content_type );
-		printf("</TD><TD ALIGN=RIGHT>%s<BR></TD></TR>",
+		printf("</label></td><td align=\"right\">%s<br /></td></tr>",
 			showsize(end_pos - start_body));
 	}
 
 	if (cnt == 0)
-		printf("<TR><TD ALIGN=CENTER>%s<BR></TD></TR>\n",
+		printf("<tr><td align=\"center\">%s<br /></td></tr>\n",
 			noattach_lab);
 	printf("</table>\n");
 }
@@ -180,7 +217,7 @@ void attachments_opts(const char *draft)
 
 	CHECKFILENAME(draft);
 
-	filename=maildir_find(DRAFTS, draft);
+	filename=maildir_find(INBOX "." DRAFTS, draft);
 	if (!filename)
 		return;
 	fp=fopen(filename, "r");
@@ -188,36 +225,39 @@ void attachments_opts(const char *draft)
 	if (!fp)
 		return;
 
-	printf("<INPUT TYPE=CHECKBOX NAME=fcc%s>%s<BR>",
-	       pref_noarchive ? "":" CHECKED",
+	printf("<label><input type=\"checkbox\" name=\"fcc\"%s />%s</label><br />",
+	       pref_noarchive ? "":" checked=\"checked\"",
 	       getarg("PRESERVELAB"));
+	if (access(NODSN, R_OK))
+		printf("<label><input type=\"checkbox\" name=\"dsn\" />%s</label><br />",
+		       getarg("DSN"));
 
 #if HAVE_SQWEBMAIL_UNICODE
-	if (has_gpg(GPGDIR) == 0)
+	if (libmail_gpg_has_gpg(GPGDIR) == 0)
 	{
 		char *all_addr;
 
-		printf("<INPUT TYPE=CHECKBOX "
-		       "NAME=sign>%s<SELECT NAME=signkey>",
+		printf("<label><input type=\"checkbox\" "
+		       "name=\"sign\" />%s</label><select name=\"signkey\">",
 		       getarg("SIGNLAB"));
 		gpgselectkey();
-		printf("</SELECT><BR>\n");
+		printf("</select><br />\n");
 
 		all_addr=newmsg_alladdrs(fp);
 
-		printf("<TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0"
-		       " VALIGN=MIDDLE><TR><TD><INPUT TYPE=CHECKBOX"
-		       " NAME=encrypt></TD><TD>%s</TD>"
-		       "<TD><SELECT SIZE=4 MULTIPLE NAME=encryptkey>",
+		printf("<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\">"
+		       "<tr valign=\"middle\"><td><input type=\"checkbox\""
+		       " name=\"encrypt\" id=\"encrypt\" /></td><td><label for=\"encrypt\">%s</label></td>"
+		       "<td><select size=\"4\" multiple=\"multiple\" name=\"encryptkey\">",
 		       getarg("ENCRYPTLAB"));
 		gpgencryptkeys(all_addr);
-		printf("</SELECT></TD>\n");
+		printf("</select></td></tr>\n");
 
 		if (ishttps())
-			printf("<TR><TD>&nbsp;</TD><TD>%s</TD><TD><input type=password name=passphrase></TD></TR>\n",
+			printf("<tr valign=\"middle\"><td>&nbsp;</td><td>%s</td><td><input type=\"password\" name=\"passphrase\" /></td></tr>\n",
 			       getarg("PASSPHRASE"));
 
-		printf("</TR></TABLE><BR>\n");
+		printf("</table><br />\n");
 		if (all_addr)
 			free(all_addr);
 	}
@@ -227,12 +267,8 @@ void attachments_opts(const char *draft)
 
 static void attachment_showname(const char *name)
 {
-char	*s;
-
 	if (!name || !*name)	name="[attachment]";	/* Eh??? */
-	s=rfc2047_decode_simple(name);
-	output_attrencoded(s ? s:name);
-	if (s)	free(s);
+	output_attrencoded(name);
 }
 
 static void attachment_open(const char *draft,
@@ -240,7 +276,7 @@ static void attachment_open(const char *draft,
 	int	*fd2,
 	struct rfc2045 **rfcp)
 {
-char	*oldname=maildir_find(DRAFTS, draft);
+char	*oldname=maildir_find(INBOX "." DRAFTS, draft);
 
 	if (!oldname)	enomem();
 
@@ -416,7 +452,7 @@ struct	stat	stat_buf;
 		enomem();
 	}
 
-	newdraftfd=maildir_recreatemsg(DRAFTS, draft, &draftfilename);
+	newdraftfd=maildir_recreatemsg(INBOX "." DRAFTS, draft, &draftfilename);
 	if (newdraftfd < 0)
 	{
 		fclose(fp);
@@ -436,14 +472,14 @@ struct	stat	stat_buf;
 	fclose(fp);
 	rfc2045_free(rfcp);
 
-	if ( maildir_closemsg(newdraftfd, DRAFTS, draftfilename, isok,
+	if ( maildir_closemsg(newdraftfd, INBOX "." DRAFTS, draftfilename, isok,
 		stat_buf.st_size))
 	{
 		free(draftfilename);
 		enomem();
 	}
 	free(draftfilename);
-	maildir_remcache(DRAFTS);	/* Cache file invalid now */
+	maildir_remcache(INBOX "." DRAFTS);	/* Cache file invalid now */
 }
 
 /* ---------------------------------------------------------------------- */
@@ -585,6 +621,23 @@ const char *r;
 }
 #endif
 
+static int cnt_filename(const char *param,
+			const char *value,
+			void *void_arg)
+{
+	*(int *)void_arg += strlen(param)+strlen(value)+5;
+	return 0;
+}
+
+static int save_filename(const char *param,
+			 const char *value,
+			 void *void_arg)
+{
+	strcat(strcat(strcat(strcat((char *)void_arg, ";\n  "), param),
+		      "="), value);
+	return 0;
+}
+
 int attach_upload(const char *draft,
 		  const char *attpubkey,
 		  const char *attprivkey)
@@ -613,7 +666,7 @@ int attach_upload(const char *draft,
 
 	/* Open the file containing the draft message */
 
-	draftfilename=maildir_find(DRAFTS, draft);
+	draftfilename=maildir_find(INBOX "." DRAFTS, draft);
 	if (!draftfilename)	return (0);
 
 	fd2=maildir_safeopen(draftfilename, O_RDONLY, 0);
@@ -640,7 +693,7 @@ int attach_upload(const char *draft,
 	** attachment
 	*/
 
-	attachfd=maildir_createmsg(DRAFTS, "temp", &attachfilename);
+	attachfd=maildir_createmsg(INBOX "." DRAFTS, "temp", &attachfilename);
 	if (attachfd < 0)
 	{
 		fclose(draftfp);
@@ -653,7 +706,7 @@ int attach_upload(const char *draft,
 	     cgi_getfiles( &upload_start, &upload_file, &upload_end, 1, NULL))
 		|| maildir_writemsg_flush(attachfd))
 	{
-		maildir_closemsg(attachfd, DRAFTS, attachfilename, 0, 0);
+		maildir_closemsg(attachfd, INBOX "." DRAFTS, attachfilename, 0, 0);
 		free(attachfilename);
 		fclose(draftfp);
 		close(attachfd);
@@ -663,8 +716,8 @@ int attach_upload(const char *draft,
 	if (fstat(attachfd, &attach_stat_buf) ||
 		attach_stat_buf.st_size + stat_buf.st_size > max_attach)
 	{
-		maildir_closemsg(attachfd, DRAFTS, attachfilename, 0, 0);
-		maildir_deletenewmsg(attachfd, DRAFTS, attachfilename);
+		maildir_closemsg(attachfd, INBOX "." DRAFTS, attachfilename, 0, 0);
+		maildir_deletenewmsg(attachfd, INBOX "." DRAFTS, attachfilename);
 		free(attachfilename);
 		fclose(draftfp);
 		close(attachfd);
@@ -709,10 +762,10 @@ int attach_upload(const char *draft,
 
 	/* Create a new version of the draft message */
 
-	newdraftfd=maildir_recreatemsg(DRAFTS, draft, &draftfilename);
+	newdraftfd=maildir_recreatemsg(INBOX "." DRAFTS, draft, &draftfilename);
 	if (newdraftfd < 0)
 	{
-		maildir_closemsg(attachfd, DRAFTS, attachfilename, 0, 0);
+		maildir_closemsg(attachfd, INBOX "." DRAFTS, attachfilename, 0, 0);
 		fclose(draftfp);
 		close(attachfd);
 		enomem();
@@ -720,8 +773,8 @@ int attach_upload(const char *draft,
 
 	if (fseek(draftfp, 0L, SEEK_SET) < 0)
 	{
-		maildir_closemsg(newdraftfd, DRAFTS, draftfilename, 0, 0);
-		maildir_closemsg(attachfd, DRAFTS, attachfilename, 0, 0);
+		maildir_closemsg(newdraftfd, INBOX "." DRAFTS, draftfilename, 0, 0);
+		maildir_closemsg(attachfd, INBOX "." DRAFTS, attachfilename, 0, 0);
 		fclose(draftfp);
 		close(attachfd);
 		enomem();
@@ -755,9 +808,9 @@ int attach_upload(const char *draft,
 			&dummy, &dummy);
 		if (messagecopy(draftfp, start_body, end_pos))
 		{
-			maildir_closemsg(newdraftfd, DRAFTS, draftfilename,
+			maildir_closemsg(newdraftfd, INBOX "." DRAFTS, draftfilename,
 				0, 0);
-			maildir_closemsg(attachfd, DRAFTS, attachfilename,
+			maildir_closemsg(attachfd, INBOX "." DRAFTS, attachfilename,
 				0, 0);
 			fclose(draftfp);
 			close(newdraftfd);
@@ -784,9 +837,9 @@ int attach_upload(const char *draft,
 					&dummy, &dummy);
 			if (messagecopy(draftfp, start_pos, end_pos))
 			{
-				maildir_closemsg(newdraftfd, DRAFTS,
+				maildir_closemsg(newdraftfd, INBOX "." DRAFTS,
 					draftfilename, 0, 0);
-				maildir_closemsg(attachfd, DRAFTS,
+				maildir_closemsg(attachfd, INBOX "." DRAFTS,
 					attachfilename, 0, 0);
 				fclose(draftfp);
 				close(newdraftfd);
@@ -799,8 +852,33 @@ int attach_upload(const char *draft,
 		}
 	}
 
-	filenamemime=rfc2047_encode_str(cgi_attachfilename,
-		sqwebmail_content_charset);
+	{
+		const char *cp=strrchr(cgi_attachfilename, '/');
+		int len;
+		static const char fnStr[]="filename";
+
+		if (cp)
+			++cp;
+		else
+			cp=cgi_attachfilename;
+
+		len=1;
+		rfc2231_attrCreate(fnStr, cp,
+				   sqwebmail_content_charset,
+				   sqwebmail_content_language,
+				   &cnt_filename, &len);
+
+		filenamemime=malloc(len);
+
+		if (filenamemime)
+		{
+			*filenamemime=0;
+			rfc2231_attrCreate(fnStr, cp,
+					   sqwebmail_content_charset,
+					   sqwebmail_content_language,
+					   save_filename, filenamemime);
+		}
+	}
 
 	argvec[0]="makemime";
 	argvec[1]="-c";
@@ -813,7 +891,7 @@ int attach_upload(const char *draft,
 	}
 	else
 	{
-		const char *pp, *f;
+		const char *pp;
 
 		argvec[2]=(char *)calc_mime_type(cgi_attachfilename);
 		n=3;
@@ -827,30 +905,20 @@ int attach_upload(const char *draft,
 		}
 
 		argvec[n++]="-N";
-		argvec[n++]=filenamemime ? filenamemime:"filename.dat";
+		argvec[n++]=cgi_attachfilename ?
+			(char *)cgi_attachfilename:"filename.dat";
 
 		pp=*cgi("attach_inline") ?
-			"Content-Disposition: inline; filename=\"":
-			"Content-Disposition: attachment; filename=\"";
+			"Content-Disposition: inline":
+			"Content-Disposition: attachment";
 
-		f=filenamemime ? filenamemime:"filename.dat";
-
-		filenamebuf=malloc(strlen(pp)+strlen(f)+10);
+		filenamebuf=malloc(strlen(pp)+strlen(filenamemime ?
+						     filenamemime:"") + 15);
 
 		if (filenamebuf)
 		{
-			char *ff;
-
 			strcpy(filenamebuf, pp);
-			ff=filenamebuf+strlen(filenamebuf);
-
-			while (*f)
-			{
-				if (*f != '"')
-					*ff++ = *f;
-				++f;
-			}
-			strcpy(ff, "\"");
+			strcat(filenamebuf, filenamemime ? filenamemime:"");
 
 			argvec[n++]="-a";
 			argvec[n++]=filenamebuf;
@@ -868,8 +936,8 @@ int attach_upload(const char *draft,
 			free(filenamemime);
 		if (filenamebuf)
 			free(filenamebuf);
-		maildir_closemsg(newdraftfd, DRAFTS, draftfilename, 0, 0);
-		maildir_closemsg(attachfd, DRAFTS, attachfilename, 0, 0);
+		maildir_closemsg(newdraftfd, INBOX "." DRAFTS, draftfilename, 0, 0);
+		maildir_closemsg(attachfd, INBOX "." DRAFTS, attachfilename, 0, 0);
 		fclose(draftfp);
 		close(newdraftfd);
 		close(attachfd);
@@ -884,8 +952,8 @@ int attach_upload(const char *draft,
 			free(filenamemime);
 		if (filenamebuf)
 			free(filenamebuf);
-		maildir_closemsg(newdraftfd, DRAFTS, draftfilename, 0, 0);
-		maildir_closemsg(attachfd, DRAFTS, attachfilename, 0, 0);
+		maildir_closemsg(newdraftfd, INBOX "." DRAFTS, draftfilename, 0, 0);
+		maildir_closemsg(attachfd, INBOX "." DRAFTS, attachfilename, 0, 0);
 		fclose(draftfp);
 		close(newdraftfd);
 		close(attachfd);
@@ -904,8 +972,8 @@ int attach_upload(const char *draft,
 		close(pipefd[0]);
 		close(pipefd[1]);
 		execv(MAKEMIME, argvec);
-		syslog(LOG_DAEMON | LOG_CRIT,
-		       "sqwebmail: %s: %s", MAKEMIME, strerror(errno));
+		fprintf(stderr,
+		       "CRIT: exec %s: %s\n", MAKEMIME, strerror(errno));
 		exit(1);
 	}
 
@@ -943,11 +1011,11 @@ int attach_upload(const char *draft,
 
 	if (waitstat > 0 || n < 0)
 	{
-		maildir_closemsg(newdraftfd, DRAFTS, draftfilename, 0, 0);
-		maildir_closemsg(attachfd, DRAFTS, attachfilename, 0, 0);
+		maildir_closemsg(newdraftfd, INBOX "." DRAFTS, draftfilename, 0, 0);
+		maildir_closemsg(attachfd, INBOX "." DRAFTS, attachfilename, 0, 0);
 		fclose(draftfp);
 		close(newdraftfd);
-		maildir_deletenewmsg(attachfd, DRAFTS, attachfilename);
+		maildir_deletenewmsg(attachfd, INBOX "." DRAFTS, attachfilename);
 		close(attachfd);
 		return (-3);
 	}
@@ -958,12 +1026,12 @@ int attach_upload(const char *draft,
 
 	/* Finish new draft message, let it replace the current one */
 
-	if (maildir_closemsg(newdraftfd, DRAFTS, draftfilename, 1,
+	if (maildir_closemsg(newdraftfd, INBOX "." DRAFTS, draftfilename, 1,
 		stat_buf.st_size))
 	{
-		maildir_closemsg(attachfd, DRAFTS, attachfilename, 0, 0);
+		maildir_closemsg(attachfd, INBOX "." DRAFTS, attachfilename, 0, 0);
 		free(draftfilename);
-		maildir_deletenewmsg(attachfd, DRAFTS, attachfilename);
+		maildir_deletenewmsg(attachfd, INBOX "." DRAFTS, attachfilename);
 		free(attachfilename);
 		rfc2045_free(rfcp);
 		fclose(draftfp);
@@ -976,7 +1044,7 @@ int attach_upload(const char *draft,
 
 	/* Remove and delete temp attachment file */
 
-	maildir_deletenewmsg(attachfd, DRAFTS, attachfilename);
+	maildir_deletenewmsg(attachfd, INBOX "." DRAFTS, attachfilename);
 	free(attachfilename);
 	rfc2045_free(rfcp);
 	return (0);
@@ -1003,7 +1071,7 @@ int	quotaflag=0;
 			tokensave();
 		}
 	}
-	else if (*cgi("uppubkey") && has_gpg(GPGDIR) == 0)
+	else if (*cgi("uppubkey") && libmail_gpg_has_gpg(GPGDIR) == 0)
 	{
 		if (!tokencheck())
 		{
@@ -1012,7 +1080,7 @@ int	quotaflag=0;
 		}
 	}
 	else if (*cgi("upprivkey") && *cgi("really") &&
-		 has_gpg(GPGDIR) == 0)
+		 libmail_gpg_has_gpg(GPGDIR) == 0)
 	{
 		if (!tokencheck())
 		{
