@@ -5,16 +5,12 @@
 
 
 /*
-** $Id: auth.c,v 1.30 2001/08/19 06:13:18 mrsam Exp $
+** $Id: auth.c,v 1.37 2004/05/09 03:32:01 mrsam Exp $
 */
 
 #include	"sqwebmail.h"
 #include	"config.h"
 
-#if	HAVE_SYSLOG_H
-#include	<syslog.h>
-#endif
-
 #if	HAVE_SYS_STAT_H
 #include	<sys/stat.h>
 #endif
@@ -38,6 +34,7 @@
 #include	"authlib/auth.h"
 #include	"authlib/authmod.h"
 #include	"authlib/authstaticlist.h"
+#include	"maildir/loginexec.h"
 
 #include	"auth.h"
 #include	"htmllibdir.h"
@@ -45,7 +42,12 @@
 
 extern int check_sqwebpass(const char *);
 
-static char *myhostname()
+#if	HAVE_SQWEBMAIL_UNICODE
+extern char *sqwebmail_content_charset, *sqwebmail_system_charset;
+#include	"unicode/unicode.h"
+#endif
+
+const char *myhostname()
 {
 char    buf[512];
 static char *my_hostname=0;
@@ -77,9 +79,12 @@ FILE	*f;
 
 static int login_maildir(const char *maildir)
 {
+	if (!maildir || !*maildir)
+		maildir=getenv("MAILDIRPATH");
 	if (!maildir || !*maildir)
 		maildir="Maildir";
 	if (chdir(maildir))	return (-1);
+	maildir_loginexec();
 	return (0);
 }
 
@@ -105,6 +110,7 @@ const char *p=authinfo->maildir;
 char	*buf;
 struct stat stat_buf;
 
+	if (!p || !*p)	p=getenv("MAILDIRPATH");
 	if (!p || !*p)	p="Maildir";
 
 	buf=malloc(strlen(h)+strlen(p)+sizeof(PASSFILE)+100);
@@ -160,7 +166,7 @@ int	rc;
 	pi.haswebpass=0;
 	pi.i=0;
 
-	rc=authstaticlist_search(u, "webmail", MODULEFILE,
+	rc=authstaticlist_search_string(u, "webmail", getenv("AUTHMODULES"),
 		&pre_callback, &pi);
 
 	if (rc)
@@ -200,12 +206,21 @@ int	rc;
 
 	if (uid)	free(uid);
 
-	uid=authlogin_search(MODULEFILE, "webmail", AUTHTYPE_LOGIN, buf,
-			     0, NULL, NULL, &i);
+	uid=authlogin_search_string(getenv("AUTHMODULES"), "webmail",
+				    AUTHTYPE_LOGIN, buf, 0, NULL, NULL, &i);
 	free(buf);
 
 	if (uid != 0)
 	{
+		char *p=authgetoptionenv("disablewebmail");
+
+		if (p && atoi(p))
+		{
+			free(p);
+			return NULL;
+		}
+		free(p);
+
 		if (login_maildir(getenv("MAILDIR")))
 			error("Unable to open the maildir for this account -- the maildir doesn't exist or has incorrect ownership or permissions.");
 
@@ -354,7 +369,7 @@ int	i;
 const char *login_returnaddr()
 {
 const char *p=getenv("AUTHADDR");
-char	*domain=myhostname();
+const char *domain=myhostname();
 
 static char *addrbuf=0;
 
@@ -378,9 +393,42 @@ char	*q;
 
 static char *hdrbuf=0;
 
+#if	HAVE_SQWEBMAIL_UNICODE
+FILE *fp;
+char authcharset[128];
+const struct unicode_info *uiptr=NULL, *uoptr=NULL;
+char *ufullname=0;
+static char *uhdrbuf=0;
+#endif
+
 	if (!fullname || !*fullname)
 		return (address);	/* That was easy */
 
+#if	HAVE_SQWEBMAIL_UNICODE
+	authcharset[0] = 0;
+	if ((fp=fopen(AUTHCHARSET, "r")))
+	{
+		char *p;
+		fgets(authcharset, sizeof(authcharset), fp);
+		fclose(fp);
+
+		if ((p=strchr(authcharset, '\n')))
+			*p = '\0';
+	}
+
+	if (authcharset[0] == 0
+	    && sqwebmail_system_charset && *sqwebmail_system_charset
+	    && strcasecmp(sqwebmail_system_charset, "ASCII"))
+		strncpy(authcharset, sqwebmail_system_charset,
+			sizeof(authcharset)-1);
+
+	if (authcharset[0] && (uiptr=unicode_find(authcharset))
+	    && sqwebmail_content_charset && *sqwebmail_content_charset
+	    && (uoptr=unicode_find(sqwebmail_content_charset))
+	    && (ufullname=unicode_ctoutf8(uiptr, fullname, NULL)))
+		fullname = ufullname;
+#endif
+
 	l=sizeof("\"\" <>")+strlen(address)+strlen(fullname);
 
 	for (p=fullname; *p; p++)
@@ -414,6 +462,12 @@ static char *hdrbuf=0;
 	*q++='>';
 	*q=0;
 
+#if	HAVE_SQWEBMAIL_UNICODE
+	if (ufullname)	free(ufullname);
+	if (uhdrbuf)	free(uhdrbuf);
+	if (uoptr && (uhdrbuf=unicode_cfromutf8(uoptr, hdrbuf, NULL)))
+		return (uhdrbuf);
+#endif
 	return (hdrbuf);
 }
 
