@@ -5,7 +5,7 @@
 
 
 /*
-** $Id: maildir.c,v 1.83 2004/05/25 02:08:41 mrsam Exp $
+** $Id: maildir.c,v 1.84 2004/08/29 19:06:56 mrsam Exp $
 */
 #include	"config.h"
 #include	<stdio.h>
@@ -1488,8 +1488,8 @@ static int subjectcmp(const char *a, const char *b)
 	as=rfc2047_decode_unicode(a, &unicode_UTF8, 0);
 	bs=rfc2047_decode_unicode(b, &unicode_UTF8, 0);
 #else
-	as=rfc2047_decode_simple(a, &unicode_UTF8);
-	bs=rfc2047_decode_simple(b, &unicode_UTF8);
+	as=rfc2047_decode_simple(a);
+	bs=rfc2047_decode_simple(b);
 #endif
 
 	if (as)
