@@ -1,11 +1,11 @@
 /*
-** Copyright 1998 - 1999 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2004 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
 
 /*
-** $Id: newmsg_create.c,v 1.41 2001/07/15 16:50:15 mrsam Exp $
+** $Id: newmsg_create.c,v 1.53 2004/05/23 14:28:24 mrsam Exp $
 */
 #include	"config.h"
 #include	"cgi/cgi.h"
@@ -13,6 +13,7 @@
 #include	"sqwebmail.h"
 #include	"auth.h"
 #include	"maildir.h"
+#include	"newmsg.h"
 #include	"folder.h"
 #include	"filter.h"
 #include	"pref.h"
@@ -23,9 +24,11 @@
 #include	"rfc2045/rfc2045.h"
 #include	"rfc2045/rfc2646.h"
 #include	"rfc822/rfc2047.h"
+#include	"rfc822/encode.h"
 #include	"gpglib/gpglib.h"
 #include	"http11/http11.h"
 #include	"htmllibdir.h"
+#include	"unicode/unicode.h"
 
 #include	<stdlib.h>
 #if HAVE_UNISTD_H
@@ -53,7 +56,7 @@ char *newmsg_createdraft(const char *curdraft)
 	if (curdraft && *curdraft)
 	{
 	char	*base=maildir_basename(curdraft);
-	char	*filename=maildir_find(DRAFTS, base);
+	char	*filename=maildir_find(INBOX "." DRAFTS, base);
 
 		if (filename)
 		{
@@ -100,16 +103,85 @@ static void create_draftheader(const char *hdrname, const char *p,
 	create_draftheader_do(hdrname, p, isrfc822addr);
 }
 
+#define	ISLWS(c)	((c)=='\t' || (c)=='\r' || (c)=='\n' || (c)==' ')
+
+static void header_wrap(const char *name, const char *hdr,
+			char *outbuf, size_t *outcnt)
+{
+char	*pfix;
+size_t	offset=strlen(name);
+
+	*outcnt=0;
+	pfix="";
+
+	while (*hdr)
+	{
+	size_t i;
+	size_t spc;
+
+		for (spc=0, i=0; hdr[i]; i++)
+		{
+			if (i + offset >= 75)
+			{
+				if (spc)
+					i = spc;
+				offset = 0;
+				break;
+			}
+
+			if (ISLWS(hdr[i]))
+			{
+				spc = i;
+				while (ISLWS(hdr[i+1]))
+					++i;
+			}
+		}
+
+		if (outbuf)
+		{
+			strcpy(outbuf, pfix);
+			outbuf += strlen(pfix);
+		}
+		*outcnt += strlen(pfix);
+
+		if (outbuf)
+		{
+		size_t j;
+			for (j=0; j < i; j++)
+			{
+				if (ISLWS(hdr[j]))
+				{
+					*(outbuf++) = ' ';
+					while (ISLWS(hdr[j+1]))
+						++j;
+				}
+				else
+					*(outbuf++) = hdr[j];
+			}
+		}
+		*outcnt += i;
+		pfix="\n  ";
+		hdr += i;
+		while (ISLWS(*hdr))
+			++hdr;
+	}
+	if (outbuf)
+		*outbuf=0;
+	++*outcnt;
+}
+			
 static void create_draftheader_do(const char *hdrname, const char *p,
 	int isrfc822addr)
 {
-char	*s;
+char	*s, *t;
+size_t	l;
 
 	if (!*p)	return;
 
 	if (!isrfc822addr)
 	{
-		s=rfc2047_encode_str(p, sqwebmail_content_charset);
+		s=rfc2047_encode_str(p, sqwebmail_content_charset,
+				     rfc2047_qp_allow_any);
 	}
 	else
 	{
@@ -134,6 +206,20 @@ char	*s;
 		}
 	}
 
+	header_wrap(hdrname, s, NULL, &l);
+	if (l)
+	{
+		if (!(t=malloc(l))) enomem();
+		header_wrap(hdrname, s, t, &l);
+		if (*t)
+		{
+			free(s);
+			s = t;
+		}
+		else
+			free(t);
+	}
+
 	if (!s)
 	{
 		close(newdraftfd);
@@ -204,11 +290,13 @@ const char *content_transfer_encoding;
 const char *charset;
 unsigned long prev_size=0;
 off_t	transferencodingpos;
+int is_newevent=strcmp(cgi("form"), "newevent") == 0;
+/* We're on the 'new event' screen */
 
 	if (curdraft)	/* Reuse a draft filename */
-		newdraftfd=maildir_recreatemsg(DRAFTS, curdraft, &draftfilename);
+		newdraftfd=maildir_recreatemsg(INBOX "." DRAFTS, curdraft, &draftfilename);
 	else
-		newdraftfd=maildir_createmsg(DRAFTS, 0, &draftfilename);
+		newdraftfd=maildir_createmsg(INBOX "." DRAFTS, 0, &draftfilename);
 	if (newdraftfd < 0)	enomem();
 
 	fp=NULL;
@@ -235,19 +323,28 @@ off_t	transferencodingpos;
 
 		while ((header=maildir_readheader(fp, &value, 1)) != NULL)
 		{
-			if (!keepheader)
+			if (keepheader == NEWMSG_SQISPELL)
+			{
+				if (strcasecmp(header, "mime-version") == 0 ||
+				    strncasecmp(header, "content-", 8) == 0)
+					continue;
+			}
+			else if (keepheader == NEWMSG_PCP)
+			{
+				if (strcasecmp(header, "mime-version") == 0 ||
+				    strncasecmp(header, "content-", 8) == 0 ||
+				    strcasecmp(header, "date") == 0 ||
+				    strcasecmp(header, "from") == 0 ||
+				    strcasecmp(header, "subject") == 0)
+					continue;
+			}
+			else
 			{
 				if (strcmp(header, "in-reply-to") &&
 					strcmp(header, "references") &&
 					strncmp(header, "x-", 2))	continue;
 				/* Do not discard these headers */
 			}
-			else	/* Called from spell */
-			{
-				if (strcasecmp(header, "mime-version") == 0 ||
-					strncasecmp(header, "content-", 8) == 0)
-					continue;
-			}
 
 			maildir_writemsgstr(newdraftfd, header);
 			maildir_writemsgstr(newdraftfd, ": ");
@@ -255,10 +352,13 @@ off_t	transferencodingpos;
 			maildir_writemsgstr(newdraftfd, "\n");
 		}
 	}
+	else if (is_newevent)
+		maildir_writemsgstr(newdraftfd, "X-Event: 1\n");
 
-	if (!keepheader)	/* Coming back from msg edit, set headers */
+	if (!keepheader
+	    || keepheader == NEWMSG_PCP)
+	/* Coming back from msg edit, set headers */
 	{
-	time_t	t;
 	const	char *p=cgi("headerfrom");
 
 		if (!*p)	p=pref_from;
@@ -273,21 +373,26 @@ off_t	transferencodingpos;
 /* sam ????
 	create_draftheader("In-Reply-To: ", cgi("headerin-reply-to"));
 */
+		if (!is_newevent)
+		{
+			create_draftheader("To: ", cgi("headerto"),
+					   cgi("addressbook_to"), 1);
+			create_draftheader("Cc: ", cgi("headercc"),
+					   cgi("addressbook_cc"), 1);
+			create_draftheader("Bcc: ", cgi("headerbcc"),
+					   cgi("addressbook_bcc"), 1);
+			create_draftheader("Reply-To: ", cgi("headerreply-to"), 0, 1);
+		}
+	}
+
+	if (!keepheader || keepheader == NEWMSG_PCP)
+	{
+	time_t	t;
 
-		create_draftheader("To: ", cgi("headerto"),
-				cgi("addressbook_to"), 1);
-		create_draftheader("Cc: ", cgi("headercc"),
-				cgi("addressbook_cc"), 1);
-		create_draftheader("Bcc: ", cgi("headerbcc"),
-				cgi("addressbook_bcc"), 1);
-		create_draftheader("Reply-To: ", cgi("headerreply-to"), 0, 1);
 		create_draftheader("Subject: ", cgi("headersubject"), 0, 0);
 
-		maildir_writemsgstr(newdraftfd, "Date: ");
-
-		time (&t);
-		maildir_writemsgstr(newdraftfd, rfc822_mkdt(t));
-		maildir_writemsgstr(newdraftfd, "\n");
+		time(&t);
+		create_draftheader("Date: ", rfc822_mkdate(t), 0, 0);
 	}
 
 	/* If the message has attachments, calculate multipart boundary */
@@ -323,7 +428,8 @@ off_t	transferencodingpos;
 			if (strcmp(content_type, "text/plain") == 0) break;
 		}
 
-		maildir_writemsgstr(newdraftfd, "\nContent-Type: text/plain");
+		maildir_writemsgstr(newdraftfd,
+				    "\nContent-Type: text/plain; format=xdraft");
 
 		if (q)
 		{
@@ -359,7 +465,7 @@ off_t	transferencodingpos;
 		}
 
 		maildir_writemsgstr(newdraftfd, "Mime-Version: 1.0\n"
-				    "Content-Type: text/plain;"
+				    "Content-Type: text/plain; format=xdraft;"
 				    " charset=\"");
 		maildir_writemsgstr(newdraftfd, charset);
 		maildir_writemsgstr(newdraftfd, "\"\n");
@@ -413,7 +519,7 @@ off_t	transferencodingpos;
 		}
 	}
 
-	if ( maildir_closemsg(newdraftfd, DRAFTS, draftfilename, -1, prev_size))
+	if ( maildir_closemsg(newdraftfd, INBOX "." DRAFTS, draftfilename, -1, prev_size))
 		cgi_put("error", "quota");
 
 	return(draftfilename);
@@ -427,7 +533,8 @@ static void do_save(const char *p, size_t l)
 /* Create message in the sent folder */
 
 static void sentmsg_copy(FILE *, struct rfc2045 *);
-static void sentmsg_reformat(FILE *, struct rfc2045 *);
+static void sentmsg_reformat(FILE *, struct rfc2045 *, int,
+			     const char *);
 
 extern void header_uc(char *h);
 
@@ -542,7 +649,13 @@ static void lookup_addressbook(const char *header, const char *value)
 {
 struct lookup_buffers *lookup_buffer_list=0;
 int	rc;
-char	*s=strdup(value);
+#if	HAVE_SQWEBMAIL_UNICODE
+const struct unicode_info *uiptr=unicode_find(sqwebmail_content_charset);
+char	*s=rfc2047_decode_unicode(value, uiptr,
+				  RFC2047_DECODE_NOTAG|RFC2047_DECODE_REPLACE);
+#else
+char	*s=rfc2047_decode_simple(value);
+#endif
 
 	if (!s)	enomem();
 	rc=lookup_addressbook_do(header, s, &lookup_buffer_list);
@@ -560,9 +673,9 @@ char	*s=strdup(value);
 	if (rc)	enomem();
 }
 
-char *newmsg_createsentmsg(const char *draftname, int *isgpgerr)
+char *newmsg_createsentmsg(const char *draftname, int *isgpgerr, int footer)
 {
-char	*filename=maildir_find(DRAFTS, draftname);
+char	*filename=maildir_find(INBOX "." DRAFTS, draftname);
 FILE	*fp;
 char	*sentname;
 char	*header, *value;
@@ -572,6 +685,7 @@ int	x;
 #if 0
 off_t	transferencodingpos;
 #endif
+const char	*transfer_encoding=NULL;
 
 	*isgpgerr=0;
  
@@ -598,7 +712,7 @@ off_t	transferencodingpos;
 		enomem();
 	}
 
-	newdraftfd=maildir_createmsg(SENT, 0, &sentname);
+	newdraftfd=maildir_createmsg(INBOX "." SENT, 0, &sentname);
 	if (newdraftfd < 0)
 	{
 		rfc2045_free(rfcp);
@@ -633,15 +747,49 @@ off_t	transferencodingpos;
 		if (strcasecmp(header, "Content-Type") == 0 &&
 		    !rfcp->firstpart)	/* Need to punt this header */
 		{
+		const char *content_type;
+		const char *content_transfer_encoding;
+		const char *charset;
+		off_t	end_pos, start_body, dummy;
+			rfc2045_mimeinfo(rfcp, &content_type,
+					 &content_transfer_encoding, &charset);
+			if (charset && *charset)
+			{
+				rfc2045_mimepos(rfcp, &dummy,
+						&end_pos, &start_body,
+						&dummy, &dummy);
+				transfer_encoding =
+					libmail_encode_autodetect_fppos(fp,
+						charset, start_body, end_pos);
+			}
+			else
+				transfer_encoding = NULL;
 			maildir_writemsgstr(newdraftfd,
-					    "Content-Type: text/plain;"
-					    " format=flowed; charset=\"");
-			maildir_writemsgstr(newdraftfd,
-					    sqwebmail_content_charset);
-			maildir_writemsgstr(newdraftfd, "\"\n");
+					    "Content-Type: text/plain");
+			if (charset && *charset)
+			{
+				maildir_writemsgstr(newdraftfd, "; charset=\"");
+				maildir_writemsgstr(newdraftfd, charset);
+				maildir_writemsgstr(newdraftfd, "\"");
+			}
+			maildir_writemsgstr(newdraftfd, "; format=flowed");
+			maildir_writemsgstr(newdraftfd, "\n");
+
+			if (transfer_encoding)
+			{
+				maildir_writemsgstr(newdraftfd,
+					"Content-Transfer-Encoding: ");
+				maildir_writemsgstr(newdraftfd,
+						    transfer_encoding);
+				maildir_writemsgstr(newdraftfd, "\n");
+			}
 			continue;
 		}
 
+		if (strcasecmp(header, "Content-Transfer-Encoding") == 0 &&
+		    !rfcp->firstpart)	/* Need to regenerate this header */
+			continue;
+
 		maildir_writemsgstr(newdraftfd, header);
 		maildir_writemsgstr(newdraftfd, ": ");
 		maildir_writemsgstr(newdraftfd, value);
@@ -663,7 +811,8 @@ off_t	transferencodingpos;
 	maildir_writemsgstr(newdraftfd, "\n");
 
 	if (!rfcp->firstpart)
-		sentmsg_reformat(fp, rfcp);
+		sentmsg_reformat(fp, rfcp, footer,
+				 transfer_encoding);
 	else
 	{
 	int	found_textplain=0;
@@ -674,6 +823,7 @@ off_t	transferencodingpos;
 		const char *content_type;
 		const char *content_transfer_encoding;
 		const char *charset;
+		off_t	end_pos, start_body, dummy;
 
 			rfc2045_mimeinfo(p, &content_type,
 				&content_transfer_encoding, &charset);
@@ -681,17 +831,43 @@ off_t	transferencodingpos;
 			if (strcmp(content_type, "text/plain") == 0 &&
 				!p->isdummy && !found_textplain)
 			{
+				if (charset && *charset)
+				{
+					rfc2045_mimepos(p, &dummy,
+							&end_pos, &start_body,
+							&dummy, &dummy);
+					transfer_encoding =
+					    libmail_encode_autodetect_fppos(fp,
+						charset, start_body, end_pos);
+				}
+				else
+					transfer_encoding = NULL;
 				maildir_writemsgstr(newdraftfd,
-						    "Content-Type: text/plain;"
-						    " format=flowed;"
-						    " charset=");
-				maildir_writemsgstr(newdraftfd, charset);
+						    "Content-Type: text/plain");
+				if (charset && *charset)
+				{
+					maildir_writemsgstr(newdraftfd,
+							    "; charset=\"");
+					maildir_writemsgstr(newdraftfd,
+							    charset);
+					maildir_writemsgstr(newdraftfd, "\"");
+				}
 				maildir_writemsgstr(newdraftfd,
-					"\nContent-Transfer-Encoding: ");
-				maildir_writemsgstr(newdraftfd,
-					content_transfer_encoding);
-				maildir_writemsgstr(newdraftfd, "\n\n");
-				sentmsg_reformat(fp, p);
+						    "; format=flowed");
+				maildir_writemsgstr(newdraftfd, "\n");
+
+				if (transfer_encoding)
+				{
+					maildir_writemsgstr(newdraftfd,
+					    "Content-Transfer-Encoding: ");
+					maildir_writemsgstr(newdraftfd,
+						transfer_encoding);
+					maildir_writemsgstr(newdraftfd, "\n");
+				}
+				maildir_writemsgstr(newdraftfd, "\n");
+
+				sentmsg_reformat(fp, p, footer,
+						 transfer_encoding);
 				found_textplain=1;
 			}
 			else	sentmsg_copy(fp, p);
@@ -722,13 +898,13 @@ off_t	transferencodingpos;
 
 	if ( maildir_writemsg_flush(newdraftfd))
 	{
-		maildir_closemsg(newdraftfd, SENT, sentname, 0, 0);
+		maildir_closemsg(newdraftfd, INBOX "." SENT, sentname, 0, 0);
 		free(sentname);
 		return (0);
 	}
 
 #if HAVE_SQWEBMAIL_UNICODE
-	if (has_gpg(GPGDIR) == 0)
+	if (libmail_gpg_has_gpg(GPGDIR) == 0)
 	{
 		char dosign= *cgi("sign");
 		char doencrypt= *cgi("encrypt");
@@ -757,7 +933,7 @@ off_t	transferencodingpos;
 
 		if (lseek(newdraftfd, 0L, SEEK_SET) < 0)
 		{
-			maildir_closemsg(newdraftfd, SENT,
+			maildir_closemsg(newdraftfd, INBOX "." SENT,
 					 sentname, 0, 0);
 			free(sentname);
 			return (0);
@@ -776,12 +952,12 @@ off_t	transferencodingpos;
 			*/
 
 			char *newnewsentname;
-			int newnewdraftfd=maildir_createmsg(SENT, 0,
+			int newnewdraftfd=maildir_createmsg(INBOX "." SENT, 0,
 							    &newnewsentname);
 
 			if (newnewdraftfd < 0)
 			{
-				maildir_closemsg(newdraftfd, SENT,
+				maildir_closemsg(newdraftfd, INBOX "." SENT,
 						 sentname, 0, 0);
 				free(sentname);
 				free(encryptkeys);
@@ -791,10 +967,10 @@ off_t	transferencodingpos;
 			if (gpgdomsg(newdraftfd, newnewdraftfd,
 				     signkey, encryptkeys))
 			{
-				maildir_closemsg(newnewdraftfd, SENT,
+				maildir_closemsg(newnewdraftfd, INBOX "." SENT,
 						 newnewsentname, 0, 0);
 				free(newnewsentname);
-				maildir_closemsg(newdraftfd, SENT,
+				maildir_closemsg(newdraftfd, INBOX "." SENT,
 						 sentname, 0, 0);
 				free(sentname);
 				free(encryptkeys);
@@ -802,7 +978,7 @@ off_t	transferencodingpos;
 				return (0);
 			}
 
-			maildir_closemsg(newdraftfd, SENT, sentname, 0, 0);
+			maildir_closemsg(newdraftfd, INBOX "." SENT, sentname, 0, 0);
 			free(sentname);
 			sentname=newnewsentname;
 			newdraftfd=newnewdraftfd;
@@ -812,7 +988,7 @@ off_t	transferencodingpos;
 	}
 #endif
 
-	if ( maildir_closemsg(newdraftfd, SENT, sentname, 1, 0))
+	if ( maildir_closemsg(newdraftfd, INBOX "." SENT, sentname, 1, 0))
 	{
 		free(sentname);
 		return (0);
@@ -862,13 +1038,21 @@ static int write_to_draft(const char *p, size_t l, void *dummy)
 	return (0);
 }
 
-static void sentmsg_reformat(FILE *f, struct rfc2045 *p)
+static int write_to_draft_encode(const char *p, size_t l, void *arg)
+{
+	libmail_encode((struct libmail_encode_info *)arg, p, l);
+	return (0);
+}
+
+static void sentmsg_reformat(FILE *f, struct rfc2045 *p, int do_footer,
+			     const char *transfer_encoding)
 {
 	off_t start_pos, end_pos, start_body;
 	char buf[BUFSIZ];
 	int n;
 	off_t	dummy;
 	FILE	*fp;
+	struct libmail_encode_info encode_info;
 	struct rfc2646create *createptr;
 
 	rfc2045_mimepos(p, &start_pos, &end_pos, &start_body,
@@ -880,7 +1064,10 @@ static void sentmsg_reformat(FILE *f, struct rfc2045 *p)
 		enomem();
 	}
 
-	createptr=rfc2646create_alloc( &write_to_draft, NULL);
+	libmail_encode_start(&encode_info, transfer_encoding, &write_to_draft,
+			     NULL);
+
+	createptr=rfc2646create_alloc( &write_to_draft_encode, &encode_info);
 	if (!createptr)
 	{
 		fclose(f);
@@ -903,16 +1090,23 @@ static void sentmsg_reformat(FILE *f, struct rfc2045 *p)
 			close(newdraftfd);
 			enomem();
 		}
+		/* else
+			libmail_encode(&encode_info, buf, n); */
 		start_body += n;
 	}
 
-	fp=http11_open_langfile(HTMLLIBDIR, sqwebmail_content_language,
-		"footer");
-	if (fp != 0)
+	if (do_footer)
 	{
-		while ((n=fread(buf, 1, sizeof(buf), fp)) > 0)
-			rfc2646create_parse(createptr, buf, n);
-		fclose(fp);
+		fp=http11_open_langfile(get_templatedir(),
+					sqwebmail_content_language,
+					"footer");
+		
+		if (fp != 0)
+		{
+			while ((n=fread(buf, 1, sizeof(buf), fp)) > 0)
+				rfc2646create_parse(createptr, buf, n);
+			fclose(fp);
+		}
 	}
 
 	if (rfc2646create_free(createptr))
@@ -922,6 +1116,7 @@ static void sentmsg_reformat(FILE *f, struct rfc2045 *p)
 		enomem();
 	}
 
+	libmail_encode_end(&encode_info);
 }
 
 /* ---------------------------------------------------------------------- */
