@@ -1,11 +1,11 @@
 /*
-** Copyright 2001 Double Precision, Inc.  See COPYING for
+** Copyright 2001-2004 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
 
 /*
-** $Id: pcp.c,v 1.14 2002/01/13 17:41:20 mrsam Exp $
+** $Id: pcp.c,v 1.26 2004/05/02 03:25:53 mrsam Exp $
 */
 #include	"sqwebmail.h"
 #include	"pcp.h"
@@ -36,13 +36,6 @@
 #include	<errno.h>
 #include	<sys/types.h>
 #include	<sys/stat.h>
-#if HAVE_SYSLOG_H
-#include <syslog.h>
-#define syslog2 syslog
-#else
-#define syslog(a,b)
-#define syslog2(a,b,c)
-#endif
 #if HAVE_SYS_WAIT_H
 #include	<sys/wait.h>
 #endif
@@ -62,6 +55,8 @@
 
 #define		CHANGEDFILE "calendar/changed"	/* Hack */
 
+#include	"strftime.h"
+
 extern FILE *open_langform(const char *lang, const char *formname,
 			   int print_header);
 
@@ -171,12 +166,12 @@ static void do_login(const char *userid, const char *password, int showerr)
 
 		pcp=pcp_open_server(userid, password, &errflag);
 
-		if (!pcp && errflag)
+		if (!pcp && errflag && showerr)
 		{
-			printf("%s<blockquote><pre class=\"small-error\">",
+			printf("%s<div class=\"indent\"><pre class=\"small-error\">",
 			       getarg("CALENDARLOGINERR"));
 			output_attrencoded_oknl_fp(errflag, stdout);
-			printf("</pre></blockquote>\n");
+			printf("</pre></div>\n");
 		}
 		if (errflag)
 			free(errflag);
@@ -363,8 +358,8 @@ static void refreshcache(struct PCP *pcp)
 		char buf1[NUMBUFSIZE], buf2[NUMBUFSIZE];
 
 		fprintf(new_fp, "%s\t%s\t%s\t%s\t%s\n",
-			str_time_t(recs[i].start, buf1),
-			str_time_t(recs[i].end, buf2),
+			libmail_str_time_t(recs[i].start, buf1),
+			libmail_str_time_t(recs[i].end, buf2),
 			recs[i].eventid,
 			recs[i].flags,
 			recs[i].subject);
@@ -437,9 +432,6 @@ static int createcache(struct PCP *pcp,
 
 	if (pcp_list_all(pcp, &la))
 	{
-		syslog2(LOG_NOTICE,
-			"pcp_list_all: error: %s", pcp_errmsg(pcp));
-
 		while ((p=list) != NULL)
 		{
 			list=p->next;
@@ -463,7 +455,7 @@ static int createcache(struct PCP *pcp,
 	a=malloc(sizeof(struct cacherecord_list *)*n);
 	if (!a)
 	{
-		syslog(LOG_NOTICE, "malloc failed");
+		fprintf(stderr, "NOTICE: malloc failed\n");
 
 		while ((p=list) != NULL)
 		{
@@ -482,7 +474,7 @@ static int createcache(struct PCP *pcp,
 
 	if ((*recs=malloc(sizeof(struct cacherecord)*n)) == NULL)
 	{
-		syslog(LOG_NOTICE, "malloc failed");
+		fprintf(stderr, "NOTICE: malloc failed\n");
 
 		while ((p=list) != NULL)
 		{
@@ -510,7 +502,7 @@ static int createcache(struct PCP *pcp,
 
 	if (!event_ids)
 	{
-		syslog(LOG_NOTICE, "malloc failed");
+		fprintf(stderr, "NOTICE: malloc failed\n");
 
 		destroycache(*recs, *n_recs);
 		return (-1);
@@ -531,7 +523,7 @@ static int createcache(struct PCP *pcp,
 
 	if (pcp_retr(pcp, &r))
 	{
-		syslog2(LOG_NOTICE, "pcp_retr: error: %s", pcp_errmsg(pcp));
+		fprintf(stderr, "NOTICE: pcp_retr: error: %s\n", pcp_errmsg(pcp));
 		free(event_ids);
 		destroycache(*recs, *n_recs);
 		return (-1);
@@ -967,21 +959,21 @@ void sqpcp_eventtimes()
 	for (i=0; i<n_event_time_list; i++)
 	{
 		if (i)
-			printf("<br>\n");
+			printf("<br />\n");
 		if (pcp_fmttimerange(buffer, sizeof(buffer),
 				     event_time_list[i].start,
 				     event_time_list[i].end))
 			continue;
-		printf("<tt>");
+		printf("<span class=\"tt\">");
 		print_safe(buffer);
-		printf("&nbsp;&nbsp;</tt><a href=\"");
+		printf("&nbsp;&nbsp;</span><a href=\"");
 		output_scriptptrget();
 		printf("&amp;form=newevent&amp;draftmessage=");
 		output_urlencoded(cgi("draftmessage"));
 		printf("&amp;do.deleventtime=%s-%s\"><font size=\"-2\">"
 		       "(%s)</font></a>",
-		       str_time_t(event_time_list[i].start, buffer),
-		       str_time_t(event_time_list[i].end, buffer+NUMBUFSIZE),
+		       libmail_str_time_t(event_time_list[i].start, buffer),
+		       libmail_str_time_t(event_time_list[i].end, buffer+NUMBUFSIZE),
 		       getarg("REMOVE")
 		       );
 	}
@@ -1007,15 +999,15 @@ void sqpcp_eventparticipants()
 
 	(void)ab_get_nameaddr(save_participant_names, my_participant_list);
 
-	printf("<table border=0>");
+	printf("<table border=\"0\">");
 
 	for (m=my_participant_list; m; m=m->next)
 	{
-		printf("<tr><td><tt>");
+		printf("<tr><td><span class=\"tt\">");
 		if (m->address)
 			ab_nameaddr_show(m->name, m->address);
 
-		printf("</tt></td><td>&nbsp;&nbsp;<a href=\"");
+		printf("</span></td><td>&nbsp;&nbsp;<a href=\"");
 
 		output_scriptptrget();
 		printf("&amp;form=newevent&amp;draftmessage=");
@@ -1032,9 +1024,9 @@ void sqpcp_eventfrom()
 {
 	if (access(NOCHANGINGFROM, 0) == 0)
 	{
-		printf("<TT>");
+		printf("<span class=\"tt\">");
 		print_safe(login_fromhdr());
-		printf("</TT>");
+		printf("</span>");
 	}
 	else
 	{
@@ -1047,9 +1039,9 @@ void sqpcp_eventfrom()
 		if (!p)
 			p="";
 
-		printf("<input type=text name=headerfrom size=60 value=\"");
+		printf("<input type=\"text\" name=\"headerfrom\" size=\"60\" value=\"");
 		output_attrencoded(p);
-		printf("\">");
+		printf("\" />");
 	}
 }
 
@@ -1057,7 +1049,7 @@ void sqpcp_eventtext()
 {
 	const char *p;
 
-	printf("<textarea name=message cols=%d rows=15 wrap=soft>",
+	printf("<textarea name=\"message\" cols=\"%d\" rows=\"15\" wrap=\"soft\">",
 	       MYLINESIZE);
 	p=cgi("draftmessage");
 
@@ -1138,7 +1130,7 @@ void sqpcp_summary()
 
 		strcpy(q, p);
 		free(p);
-		strcat(q, "&amp;form=eventdaily&amp;clearcache=1&date=");
+		strcat(q, "&amp;form=eventdaily&amp;clearcache=1&amp;date=");
 		sprintf(q+strlen(q), "%04d%02d%02d",
 			tmptr->tm_year + 1900,
 			tmptr->tm_mon+1,
@@ -1216,15 +1208,15 @@ void sqpcp_summary()
 				{
 					/* Same day as last event */
 
-					printf("<tr><td align=left width=\"1%%\">&nbsp;</td>"
-					       "<td align=left>&nbsp;"
+					printf("<tr><td align=\"left\" width=\"1%%\">&nbsp;</td>"
+					       "<td align=\"left\">&nbsp;"
 					       "<a href=\"");
 					print_event_link_url(cr.eventid, "");
 					printf("&amp;date=%s\">", yyyymmdd);
 					print_safe(time1);
 					printf("&nbsp;-&nbsp;");
 					print_safe(time2);
-					printf("</a>");
+					printf("</a></td>");
 					i=1;
 				}
 				else
@@ -1242,12 +1234,12 @@ void sqpcp_summary()
 			if (!i)
 			{
 				if (cnt)
-					printf("<tr><td style=\"line-height: 0px; font-size: 8px\" colspan=3>"
+					printf("<tr><td style=\"line-height: 0px; font-size: 8px\" colspan=\"3\">"
 					       "&nbsp;"
 					       "</td></tr>\n");
 				if (strcmp(date, date2))
 				{
-					printf("<tr><td colspan=2>"
+					printf("<tr><td colspan=\"2\">"
 					       "<a href=\"");
 					print_event_link_url(cr.eventid, "");
 					printf("&amp;date=%s\">", yyyymmdd);
@@ -1291,7 +1283,7 @@ void sqpcp_summary()
 
 	if (cnt == 0)
 	{
-		printf("<tr><td align=center>%s</td></tr>\n",
+		printf("<tr><td align=\"center\">%s</td></tr>\n",
 		       getarg("NOEVENTS"));
 	}
 
@@ -1352,7 +1344,7 @@ static FILE *openoldfp(const char *p, unsigned long *prev_size)
 
 	CHECKFILENAME(p);
 
-	filename=maildir_find(DRAFTS, p);
+	filename=maildir_find(INBOX "." DRAFTS, p);
 	if (!filename)
 		return (NULL);
 
@@ -1463,19 +1455,19 @@ static void show_conflict_error(struct PCP *pcp)
 
 	if (n == 0 || pcp_retr(pcp, &r) == 0)
 	{
-		printf("<table border=0 width=\"100%%\" class=\"small-error\">");
+		printf("<table border=\"0\" width=\"100%%\" class=\"small-error\">");
 		for (p=conflict_list; p; p=p->next)
 		{
 			char buffer[512];
 
-			printf("<tr><td width=30>&nbsp;</td><td><tt>");
+			printf("<tr><td width=\"30\">&nbsp;</td><td><span class=\"tt\">");
 			if (pcp_fmttimerange(buffer, sizeof(buffer),
 					     p->start,
 					     p->end) == 0)
 			{
 				print_safe(buffer);
 			}
-			printf("</tt></td><td width=30>&nbsp;</td><td width=\"100%%\"><tt>");
+			printf("</span></td><td width=\"30\">&nbsp;</td><td width=\"100%%\"><span class=\"tt\">");
 			if (p->address && strcmp(p->address, "@"))
 			{
 				printf(getarg("CONFLICTERR2"));
@@ -1484,9 +1476,9 @@ static void show_conflict_error(struct PCP *pcp)
 			else
 				print_event_subject("", p->subject
 						    ? p->subject:"", 60);
-			printf("</tt></td></tr>\n");
+			printf("</span></td></tr>\n");
 		}
-		printf("<tr><td colspan=4><hr width=\"90%%\"></td></tr></table>\n");
+		printf("<tr><td colspan=\"4\"><hr width=\"90%%\" /></td></tr></table>\n");
 	}
 	free(l);
 	init_save_conflict();
@@ -1684,11 +1676,11 @@ void sqpcp_newevent()
 	{
 		if (*cgi("do.neweventpreview") && p && *p)
 		{
-			printf("<TABLE WIDTH=\"100%%\" BORDER=0 CELLSPACING=0 CELLPADDING=1 CLASS=\"box-small-outer\"><TR><TD>\n");
-			printf("<TABLE WIDTH=\"100%%\" BORDER=0 CELLSPACING=0 CELLPADDING=4 CLASS=\"preview\"><TR><TD>\n");
+			printf("<table width=\"100%%\" border=\"0\" cellspacing=\"0\" cellpadding=\"1\" class=\"box-small-outer\"><tr><td>\n");
+			printf("<table width=\"100%%\" border=\"0\" cellspacing=\"0\" cellpadding=\"4\" class=\"preview\"><tr><td>\n");
 			newmsg_preview(p);
-			printf("</TD></TR></TABLE>\n");
-			printf("</TD></TR></TABLE>\n");
+			printf("</td></tr></table>\n");
+			printf("</td></tr></table>\n");
 		}
 		return;
 	}
@@ -1702,11 +1694,11 @@ void sqpcp_newevent()
 
 	if (p && *p)
 	{
-		newdraftfd=maildir_recreatemsg(DRAFTS, p, &draftfilename);
+		newdraftfd=maildir_recreatemsg(INBOX "." DRAFTS, p, &draftfilename);
 	}
 	else
 	{
-		newdraftfd=maildir_createmsg(DRAFTS, 0, &draftfilename);
+		newdraftfd=maildir_createmsg(INBOX "." DRAFTS, 0, &draftfilename);
 		maildir_writemsgstr(newdraftfd, "X-Event: 1\n");
 
 		if (draftmessage_buf)
@@ -1852,9 +1844,9 @@ void sqpcp_newevent()
 
 	if (errflag)
 	{
-		maildir_deletenewmsg(newdraftfd, DRAFTS, draftfilename);
+		maildir_deletenewmsg(newdraftfd, INBOX "." DRAFTS, draftfilename);
 	}
-	else if (maildir_closemsg(newdraftfd, DRAFTS, draftfilename, 1,
+	else if (maildir_closemsg(newdraftfd, INBOX "." DRAFTS, draftfilename, 1,
 				  prev_size))
 	{
 		printf("%s", getarg("QUOTAERR"));
@@ -1986,9 +1978,9 @@ static int savetime(time_t from, time_t to, void *dummy)
 	int fd= *(int *)dummy;
 
 	maildir_writemsgstr(fd, "X-Event-Time: ");
-	maildir_writemsgstr(fd, str_time_t(from, buf));
+	maildir_writemsgstr(fd, libmail_str_time_t(from, buf));
 	maildir_writemsgstr(fd, " ");
-	maildir_writemsgstr(fd, str_time_t(to, buf));
+	maildir_writemsgstr(fd, libmail_str_time_t(to, buf));
 	maildir_writemsgstr(fd, "\n");
 	return (0);
 }
@@ -2051,7 +2043,7 @@ static char *savedraft()
 		CHECKFILENAME(p);
 	}
 
-	filename=p && *p ? maildir_find(DRAFTS, p):NULL;
+	filename=p && *p ? maildir_find(INBOX "." DRAFTS, p):NULL;
 
 	msg=newmsg_createdraft_do(filename, cgi("message"), NEWMSG_PCP);
 	if (filename)
@@ -2066,7 +2058,7 @@ static void previewdraft(char *msg, void (*func)(const char *))
 {
 	char *msg2, *msg2p;
 
-	msg2=maildir_find(DRAFTS, msg);
+	msg2=maildir_find(INBOX "." DRAFTS, msg);
 	free(msg);
 	if (!msg2)
 		enomem();
@@ -2348,9 +2340,6 @@ static void free_saveinfo(struct saveinfo *si)
 static void dropquota(const char *filename, int fd)
 {
 	unsigned long filesize=0;
-	char quotabuf[QUOTABUFSIZE];
-	int quotafd;
-	int rc;
 
 	if (maildir_parsequota(filename, &filesize))
 	{
@@ -2361,27 +2350,7 @@ static void dropquota(const char *filename, int fd)
 		filesize=stat_buf.st_size;
 	}
 
-	if (maildir_getquota(".", quotabuf))
-	{
-		if (errno != ENOENT)
-			enomem();
-		return;
-	}
-
-	do
-	{
-		rc=maildir_checkquota(".", &quotafd,
-				      quotabuf, -filesize, -1);
-		if (rc && errno != EAGAIN)
-		{
-			if (quotafd >= 0)
-				close(quotafd);
-			enomem();
-		}
-	} while (rc);
-
-	maildir_addquota(".", quotafd, quotabuf, -filesize, -1);
-	if (quotafd >= 0)	close(quotafd);
+	maildir_quota_deleted(".", -filesize, -1);
 }
 
 /* ------------------------------------------------------------------------
@@ -2434,7 +2403,7 @@ void sqpcp_save()
 
 	if (sentmsg)
 	{
-		p=maildir_find(SENT, sentmsg);
+		p=maildir_find(INBOX "." SENT, sentmsg);
 		free(sentmsg);
 		sentmsg=p;
 	}
@@ -2473,7 +2442,7 @@ void sqpcp_save()
 	}
 	fclose(fp);
 
-	p=maildir_find(DRAFTS, msg);
+	p=maildir_find(INBOX "." DRAFTS, msg);
 	free(msg);
 
 	fp=p ? fopen(p, "r"):NULL;
@@ -2563,7 +2532,7 @@ static void proxy_update_list_save(const char *action,
 	{
 		if (e)
 			free(e);
-		syslog(LOG_CRIT, "sqwebmail: out of memory.");
+		fprintf(stderr, "CRIT: out of memory.\n");
 		return;
 	}
 
@@ -2628,7 +2597,7 @@ static void proxy_notify_email_msg(FILE *f, struct proxy_list_entry *l,
 	if (fseek(f, 0L, SEEK_SET) < 0
 	    || lseek(fileno(f), 0L, SEEK_SET) < 0)
 	{
-		syslog(LOG_CRIT, "sqwebmail: seek failed: %m");
+		fprintf(stderr, "CRIT: seek failed: %s\n", strerror(errno));
 		return;
 	}
 
@@ -2653,7 +2622,7 @@ static void proxy_notify_email_msg(FILE *f, struct proxy_list_entry *l,
 	if ((tmpfp=open_langform(sqwebmail_content_language, template, 0))
 	    == NULL)
 	{
-		syslog2(LOG_CRIT, "sqwebmail: %s: %m", template);
+		fprintf(stderr, "CRIT: %s: %s\n", template, strerror(errno));
 		return;
 	}
 
@@ -2662,7 +2631,7 @@ static void proxy_notify_email_msg(FILE *f, struct proxy_list_entry *l,
 	if (pipe(pipefd) < 0)
 	{
 		fclose(tmpfp);
-		syslog(LOG_CRIT, "sqwebmail: pipe: %m");
+		fprintf(stderr, "CRIT: pipe: %s\n", strerror(errno));
 		return;
 	}
 
@@ -2673,7 +2642,7 @@ static void proxy_notify_email_msg(FILE *f, struct proxy_list_entry *l,
 		close(pipefd[0]);
 		close(pipefd[1]);
 		fclose(tmpfp);
-		syslog(LOG_CRIT, "sqwebmail: fork: %m");
+		fprintf(stderr, "CRIT: fork: %s\n", strerror(errno));
 		return;
 	}
 
@@ -2685,13 +2654,13 @@ static void proxy_notify_email_msg(FILE *f, struct proxy_list_entry *l,
 		close(pipefd[1]);
 		execl(SENDITSH, "sendit.sh", returnaddr,
                                 sqwebmail_mailboxid, NULL);
-		syslog(LOG_CRIT, "sqwebmail: exec " SENDITSH ": %m");
+		fprintf(stderr, "CRIT: exec " SENDITSH ": %s\n", strerror(errno));
 		exit(1);
 	}
 	close(pipefd[0]);
 	if ((tofp=fdopen(pipefd[1], "w")) == NULL)
 	{
-		syslog(LOG_CRIT, "sqwebmail: exec " SENDITSH ": %m");
+		fprintf(stderr, "CRIT: exec " SENDITSH ": %s\n", strerror(errno));
 	}
 	else
 	{
@@ -2706,7 +2675,7 @@ static void proxy_notify_email_msg(FILE *f, struct proxy_list_entry *l,
 		;
 
 	if (!WIFEXITED(waitstat) || WEXITSTATUS(waitstat))
-		syslog(LOG_CRIT, "sqwebmail: event notify mail failed");
+		fprintf(stderr, "CRIT: event notify mail failed\n");
 }
 
 /*
@@ -2918,7 +2887,7 @@ void sqpcp_daily_view()
 
 	if ((pcp=sqpcp_calendar()) == NULL)
 	{
-		printf("<font class=\"error\">%s</font>", strerror(errno));
+		printf("<span class=\"error\">%s</span>", strerror(errno));
 		return;
 	}
 
@@ -2938,11 +2907,11 @@ static void print_event_link_url(const char *id, const char *extra)
 	printf("%s&amp;form=eventshow&amp;eventid=", extra);
 	output_urlencoded(id);
 	if (*cgi("date"))
-		printf("&date=%s", cgi("date"));
+		printf("&amp;date=%s", cgi("date"));
 	if (*cgi("weekof"))
-		printf("&weekof=%s", cgi("weekof"));
+		printf("&amp;weekof=%s", cgi("weekof"));
 	if (*cgi("monthof"))
-		printf("&monthof=%s", cgi("monthof"));
+		printf("&amp;monthof=%s", cgi("monthof"));
 }
 
 static void print_event_link(const char *id, const char *extra,
@@ -3001,7 +2970,7 @@ static void do_daily_view(struct cacherecord *recs, unsigned n_recs,
 					     start, end))
 				continue;
 
-			printf("<tr><td align=left>");
+			printf("<tr><td align=\"left\">");
 			print_event_link(recs[i].eventid, "", "class=\"dailyeventtimes\"");
 			print_safe(timerange);
 		}
@@ -3018,7 +2987,7 @@ static void do_daily_view(struct cacherecord *recs, unsigned n_recs,
 					FMTTIME_TIME))
 				continue;
 
-			printf("<tr><td align=left>");
+			printf("<tr><td align=\"left\">");
 			print_event_link(recs[i].eventid, "", "class=\"dailyeventtimes\"");
 			print_safe(time1);
 			printf("&nbsp;-&nbsp;");
@@ -3031,7 +3000,7 @@ static void do_daily_view(struct cacherecord *recs, unsigned n_recs,
 			printf("</td><td width=\"100%%\">");
 		}
 		else
-			printf("<br>");
+			printf("<br />");
 
 		printf("&nbsp;&nbsp;");
 		print_event_link(recs[i].eventid, "", "class=\"dailyeventsubject\"");
@@ -3039,14 +3008,14 @@ static void do_daily_view(struct cacherecord *recs, unsigned n_recs,
 				    viewtype == VIEW_DAILY ? 80:15);
 		printf("</a>");
 		if (viewtype != VIEW_DAILY)
-			printf("<br>&nbsp;");
+			printf("<br />&nbsp;");
 
 		printf("</td></tr>\n");
 	}
 
 	if (!printed)
 	{
-		printf("<tr><td align=center>%s</td></tr>",
+		printf("<tr><td align=\"center\">%s</td></tr>",
 		       getarg("NOEVENTS"));
 	}
 
@@ -3081,7 +3050,7 @@ void sqpcp_prevday()
 
 	printf("<a class=\"dailynextprev\" href=\"");
 	output_scriptptrget();
-	printf("&form=eventdaily&date=%04d%02d%02d\">%s</a>", y, m, d, buf);
+	printf("&amp;form=eventdaily&amp;date=%04d%02d%02d\">%s</a>", y, m, d, buf);
 }
 
 void sqpcp_nextday()
@@ -3112,7 +3081,7 @@ void sqpcp_nextday()
 
 	printf("<a class=\"dailynextprev\" href=\"");
 	output_scriptptrget();
-	printf("&form=eventdaily&date=%04d%02d%02d\">%s</a>", y, m, d, buf);
+	printf("&amp;form=eventdaily&amp;date=%04d%02d%02d\">%s</a>", y, m, d, buf);
 }
 
 /* -------------- Display event stuff --------------- */
@@ -3262,13 +3231,10 @@ void sqpcp_displayevent()
 	const char *event_id_list[2];
 	struct PCP_retr r;
 	struct display_retr dr;
-
+	struct maildir_tmpcreate_info createInfo;
 	struct display_retr_time_list *tl;
 	struct display_retr_participant_list *pl;
 
-	char *tptr, *nptr;
-	int nfd;
-
 	if (!pcp)
 		return;
 
@@ -3283,46 +3249,32 @@ void sqpcp_displayevent()
 	r.callback_retr_date=save_displayed_date;
 	r.callback_retr_participants=save_displayed_participants;
 
-	for (;;)
+	maildir_purgemimegpg(); /* Delete previous :calendar: file */
+
+	maildir_tmpcreate_init(&createInfo);
+	createInfo.uniq=":calendar:";
+	createInfo.doordie=1;
+
+	if ((dr.f=maildir_tmpcreate_fp(&createInfo)) == NULL)
 	{
-		int rc=maildir_try_create(".", ":calendar:", 0,
-					  &tptr, &nptr);
-
-		if (rc < 0)
-		{
-			error("Can't create new file.");
-		}
-		if (rc == 0)
-			break;
-		sleep(5);
-	}
-	free(nptr);
-
-	nfd=maildir_semisafeopen(tptr, O_RDWR|O_CREAT|O_TRUNC,
-				 0600);
-
-
-	if (nfd < 0 || (dr.f=fdopen(nfd, "w")) == NULL)
-	{
-		if (nfd >= 0)
-			close(nfd);
-		free(tptr);
-		enomem();
+		error(strerror(errno));
 	}
 
-	cgi_put(MIMEGPGFILENAME, strrchr(tptr, '/')+1);
+	cgi_put(MIMEGPGFILENAME, strrchr(createInfo.tmpname, '/')+1);
 
 	if (pcp_retr(pcp, &r))
 	{
 		free_display_retr(&dr);
-		free(tptr);
 		fclose(dr.f);
+		cgi_put(MIMEGPGFILENAME, "");
+		unlink(createInfo.tmpname);
+		maildir_tmpcreate_free(&createInfo);
 		show_pcp_errmsg(pcp_errmsg(pcp));
 		return;
 	}
 	fclose(dr.f);
 
-	printf("<table class=calendarevent align=center border=0><tr valign=top><th align=left>%s</th><td>",
+	printf("<table class=\"calendarevent\" align=\"center\" border=\"0\"><tr valign=\"top\"><th align=\"left\">%s</th><td>",
 	       getarg("EVENT"));
 
 	for (tl=dr.time_list; tl; tl=tl->next)
@@ -3333,27 +3285,29 @@ void sqpcp_displayevent()
 				     tl->start, tl->end))
 			continue;
 
-		printf("<tt>");
+		printf("<span class=\"tt\">");
 		print_safe(buffer);
-		printf("</tt><br>\n");
+		printf("</span><br />\n");
 	}
 	printf("</td></tr>\n");
 
 	if (dr.participant_list)
 	{
-		printf("<tr valign=top><th align=left>%s</th></td><td>",
+		printf("<tr valign=\"top\"><th align=\"left\">%s</th><td>",
 		       getarg("PARTICIPANTS"));
 		for (pl=dr.participant_list; pl; pl=pl->next)
 		{
-			printf("<tt>&lt;");
+			printf("<span class=\"tt\">&lt;");
 			print_safe(pl->participant);
-			printf("&gt;</tt><br>\n");
+			printf("&gt;</span><br />\n");
 		}
+		printf("</td></tr>");
 	}
 	printf("</table>\n");
-	folder_showmsg(DRAFTS, 0);
+	folder_showmsg(INBOX "." DRAFTS, 0);
 	free_display_retr(&dr);
-	free(tptr);
+	cgi_put(MIMEGPGFILENAME, "");
+	maildir_tmpcreate_free(&createInfo);
 }
 
 static int save_displayed_event(struct PCP_retr *r,
@@ -3518,12 +3472,12 @@ int sqpcp_eventedit()
 	if (!pcp)
 		return (-1);
 
-	newdraftfd=maildir_createmsg(DRAFTS, 0, &draftfilename);
+	newdraftfd=maildir_createmsg(INBOX "." DRAFTS, 0, &draftfilename);
 	if (doeventedit(pcp, newdraftfd))
 	{
-		maildir_deletenewmsg(newdraftfd, DRAFTS, draftfilename);
+		maildir_deletenewmsg(newdraftfd, INBOX "." DRAFTS, draftfilename);
 	}
-	else if (maildir_closemsg(newdraftfd, DRAFTS, draftfilename, 1, 0))
+	else if (maildir_closemsg(newdraftfd, INBOX "." DRAFTS, draftfilename, 1, 0))
 	{
 		cgi_put("error", "quota");
 	}
@@ -3558,9 +3512,9 @@ static int get_date(struct PCP_retr *r, time_t start, time_t end, void *vp)
 	struct getedit_info *ge=(struct getedit_info *)vp;
 
 	maildir_writemsgstr(ge->fd, "X-Event-Time: ");
-	maildir_writemsgstr(ge->fd, str_time_t(start, buf));
+	maildir_writemsgstr(ge->fd, libmail_str_time_t(start, buf));
 	maildir_writemsgstr(ge->fd, " ");
-	maildir_writemsgstr(ge->fd, str_time_t(end, buf));
+	maildir_writemsgstr(ge->fd, libmail_str_time_t(end, buf));
 	maildir_writemsgstr(ge->fd, "\n");
 
 	return (0);
@@ -3870,11 +3824,11 @@ void sqpcp_displayweek()
 		return;
 	}
 
-	printf("<table align=center border=0 class=\"weekly-border\""
-	       " cellpadding=0 cellspacing=0 width=\"100%%\">"
+	printf("<table align=\"center\" border=\"0\" class=\"weekly-border\""
+	       " cellpadding=\"0\" cellspacing=\"0\" width=\"100%%\">"
 	       "<tr><td>\n");
-	printf("<table border=1 class=\"weekly-bg\" cellspacing=0 cellpadding=10 width=\"100%%\">"
-	       "<tr valign=top>");
+	printf("<table border=\"1\" class=\"weekly-bg\" cellspacing=\"0\" cellpadding=\"10\" width=\"100%%\">"
+	       "<tr valign=\"top\">");
 
 	start=save_start;
 	for (i=0; i<7; i++)
@@ -3884,7 +3838,7 @@ void sqpcp_displayweek()
 
 		printf("<td width=\"14%%\">");
 		p=pcp_wdayname((i + pref_startofweek) % 7);
-		printf("<div align=center class=\"weekly-day\">");
+		printf("<div align=\"center\" class=\"weekly-day\">");
 		printf("<a href=\"");
 		output_scriptptrget();
 		printf("&amp;form=eventdaily&amp;date=");
@@ -3899,7 +3853,7 @@ void sqpcp_displayweek()
 		}
 		printf("\">");
 		print_safe(p);
-		printf("</a><hr width=\"70%%\">");
+		printf("</a><hr width=\"70%%\" />");
 		printf("</div>\n");
 
 		end=start;
@@ -4043,15 +3997,15 @@ void sqpcp_displaymonth()
 		return;
 	}
 
-	printf("<table align=center border=0 class=\"monthly-border\""
-	       " cellpadding=0 cellspacing=0 width=\"100%%\">"
+	printf("<table align=\"center\" border=\"0\" class=\"monthly-border\""
+	       " cellpadding=\"0\" cellspacing=\"0\" width=\"100%%\">"
 	       "<tr><td>\n");
-	printf("<table border=1 class=\"monthly-bg\" cellspacing=0 cellpadding=10 width=\"100%%\">"
-	       "<tr valign=top>");
+	printf("<table border=\"1\" class=\"monthly-bg\" cellspacing=\"0\" cellpadding=\"10\" width=\"100%%\">"
+	       "<tr valign=\"top\">");
 
 	for (i=0; i<7; i++)
 	{
-		printf("<td width=\"14%%\" align=center class=\"monthly-day\">");
+		printf("<td width=\"14%%\" align=\"center\" class=\"monthly-day\">");
 		print_safe(pcp_wdayname((i + pref_startofweek) % 7));
 		printf("</td>");
 	}
@@ -4059,7 +4013,7 @@ void sqpcp_displaymonth()
 
 	while (start < end)
 	{
-		printf("<tr valign=top>\n");
+		printf("<tr valign=\"top\">\n");
 
 		for (i=0; i<7; i++)
 		{
@@ -4091,8 +4045,7 @@ void sqpcp_displaymonth()
 			printf("\" class=\"monthly-day\">");
 
 			printf("%2d", tmptr->tm_mday);
-			printf("</a><div align=center><hr width=\"70%%\"></div>");
-			printf("</div>\n");
+			printf("</a><div align=\"center\"><hr width=\"70%%\" /></div>\n");
 
 			next_day=start;
 			nextday(&next_day);
@@ -4106,7 +4059,7 @@ void sqpcp_displaymonth()
 		printf("</tr>\n");
 	}
 	destroycache(recs, n_recs);
-	printf("</tr></table>\n");
+	printf("</table>\n");
 	printf("</td></tr></table>\n");
 }
 
@@ -4209,15 +4162,15 @@ void sqpcp_eventacl()
 	}
 	else if (acl_list)
 	{
-		printf("<table align=center>\n");
+		printf("<table align=\"center\">\n");
 
 		for (pp=acl_list; pp; pp=pp->next)
 		{
-			printf("<tr><td align=right><tt>");
+			printf("<tr><td align=\"right\"><span class=\"tt\">");
 			if (pp->addr)
 				ab_nameaddr_show(pp->name, pp->addr);
 
-			printf("</tt></td><td>-");
+			printf("</span></td><td>-");
 			if (pp->flags & PCP_ACL_MODIFY)
 				printf("&nbsp;%s", getarg("MODIFY"));
 			if (pp->flags & PCP_ACL_CONFLICT)
