@@ -1,5 +1,5 @@
 /*
-** $Id: pref.h,v 1.10 2001/03/15 04:07:06 mrsam Exp $
+** $Id: pref.h,v 1.11 2001/11/01 01:49:31 mrsam Exp $
 */
 #ifndef	pref_h
 #define	pref_h
@@ -19,6 +19,7 @@ extern void pref_pagesize();
 extern void pref_displayhtml();
 extern void pref_displayflowedtext();
 extern void pref_displaynoarchive();
+extern void pref_displayweekstart();
 
 extern int pref_flagisoldest1st, pref_flagfullheaders;
 extern int pref_flagsortorder;
@@ -27,6 +28,7 @@ extern int pref_autopurge;
 extern int pref_showhtml;
 extern int pref_noflowedtext;
 extern int pref_noarchive;
+extern int pref_startofweek;
 
 extern char *pref_from;
 extern char *pref_ldap;
