@@ -5,7 +5,7 @@
 
 
 /*
-** $Id: token.c,v 1.5 2000/07/30 01:08:10 mrsam Exp $
+** $Id: token.c,v 1.7 2003/09/22 21:44:41 mrsam Exp $
 */
 #include	"sqwebmail.h"
 #include	"config.h"
@@ -33,7 +33,7 @@ void tokennew()
 time_t	t;
 
 	time(&t);
-	printf("<INPUT TYPE=HIDDEN NAME=\"msgtoken\" VALUE=\"%ld-%ld\">",
+	printf("<input type=\"hidden\" name=\"msgtoken\" value=\"%ld-%ld\" />",
 		(long)t, (long)getpid());
 }
 
@@ -42,7 +42,7 @@ void tokennewget()
 time_t	t;
 
 	time(&t);
-	printf("&msgtoken=%ld-%ld", (long)t, (long)getpid());
+	printf("&amp;msgtoken=%ld-%ld", (long)t, (long)getpid());
 }
 
 int tokencheck()
@@ -50,6 +50,8 @@ int tokencheck()
 const char *token=cgi("msgtoken");
 const char *savedtoken;
 
+	if (!token || token[0] == '\0')
+		return(0);
 	savedtoken=read_sqconfig(".", TOKENFILE, 0);
 	if (savedtoken && strcmp(token, savedtoken) == 0)
 		return (-1);
